@@ -1,22 +1,22 @@
-/*  $Revision: 1.10 $
+/*  $Id: buffchan.c 8418 2009-04-12 21:08:22Z iulius $
 **
 **  Buffered file exploder for innd.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
-#include <sys/stat.h>
 #include <ctype.h>
 #include <errno.h>
-#include "paths.h"
-#include "libinn.h"
-#include "qio.h"
-#include "macros.h"
-#include <syslog.h>  
+#include <signal.h>
+#include <sys/stat.h>
+#include <time.h>
 
-extern void	MAPread();
-extern char	*MAPname();
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/qio.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
+#include "map.h"
 
 /*
 **  Hash functions for hashing sitenames.
@@ -31,8 +31,8 @@ extern char	*MAPname();
 **  Entry for a single active site.
 */
 typedef struct _SITE {
-    BOOL	Dropped;
-    STRING	Name;
+    bool	Dropped;
+    const char	*Name;
     int		CloseLines;
     int		FlushLines;
     time_t	LastFlushed;
@@ -40,7 +40,7 @@ typedef struct _SITE {
     int 	CloseSeconds;
     int		FlushSeconds;
     FILE	*F;
-    STRING	Filename;
+    const char	*Filename;
     char	*Buffer;
 } SITE;
 
@@ -56,42 +56,29 @@ typedef struct _SITEHASH {
 
 
 /* Global variables. */
-STATIC char	*Format;
-STATIC STRING	Map;
-STATIC int	BufferMode;
-STATIC int	CloseEvery;
-STATIC int	FlushEvery;
-STATIC int	CloseSeconds;
-STATIC int	FlushSeconds;
-STATIC SIGVAR	GotInterrupt;
-STATIC SITEHASH	SITEtable[SITE_SIZE];
-STATIC TIMEINFO	Now;
-
-
-#if !defined(HAVE_FCHMOD)
-/*
-**  A dummy fchmod.
-*/
-/* ARGSUSED */
-int
-fchmod(i)
-    int		i;
-{
-}
-#endif	/* !defined(HAVE_FCHMOD) */
+static char	*Format;
+static const char *Map;
+static int	BufferMode;
+static int	CloseEvery;
+static int	FlushEvery;
+static int	CloseSeconds;
+static int	FlushSeconds;
+static sig_atomic_t	GotInterrupt;
+static SITEHASH	SITEtable[SITE_SIZE];
+static time_t	Now;
 
 
 /*
 **  Set up the site information.  Basically creating empty buckets.
 */
-STATIC void
-SITEsetup()
+static void
+SITEsetup(void)
 {
-    register SITEHASH	*shp;
+    SITEHASH	*shp;
 
-    for (shp = SITEtable; shp < ENDOF(SITEtable); shp++) {
+    for (shp = SITEtable; shp < ARRAY_END(SITEtable); shp++) {
 	shp->Size = 3;
-	shp->Sites = NEW(SITE, shp->Size);
+	shp->Sites = xmalloc(shp->Size * sizeof(SITE));
 	shp->Used = 0;
     }
 }
@@ -100,18 +87,16 @@ SITEsetup()
 /*
 **  Close a site
 */
-STATIC void
-SITEclose(sp)
-    register SITE       *sp;
+static void
+SITEclose(SITE *sp)
 {
-    register FILE	*F;
+    FILE	*F;
 
     if ((F = sp->F) != NULL) {
 	if (fflush(F) == EOF || ferror(F)
 	 || fchmod((int)fileno(F), 0664) < 0
 	 || fclose(F) == EOF)
-	    (void)fprintf(stderr, "buffchan %s cant close %s, %s\n",
-		    sp->Name, sp->Filename, strerror(errno));
+            syswarn("%s cannot close %s", sp->Name, sp->Filename);
 	sp->F = NULL;
     }
 }
@@ -119,14 +104,14 @@ SITEclose(sp)
 /*
 **  Close all open sites.
 */
-STATIC void
-SITEcloseall()
+static void
+SITEcloseall(void)
 {
-    register SITEHASH	*shp;
-    register SITE	*sp;
-    register int	i;
+    SITEHASH	*shp;
+    SITE	*sp;
+    int	i;
 
-    for (shp = SITEtable; shp < ENDOF(SITEtable); shp++)
+    for (shp = SITEtable; shp < ARRAY_END(SITEtable); shp++)
 	for (sp = shp->Sites, i = shp->Used; --i >= 0; sp++)
 	    SITEclose(sp);
 }
@@ -135,25 +120,20 @@ SITEcloseall()
 /*
 **  Open the file for a site.
 */
-STATIC void SITEopen(SITE *sp)
+static void SITEopen(SITE *sp)
 {
     int			e;
 
     if ((sp->F = xfopena(sp->Filename)) == NULL
      && ((e = errno) != EACCES || chmod(sp->Filename, 0644) < 0
       || (sp->F = xfopena(sp->Filename)) == NULL)) {
-	(void)fprintf(stderr, "buffchan %s cant fopen %s, %s\n",
-		sp->Name, sp->Filename, strerror(e));
-	if ((sp->F = fopen("/dev/null", "w")) == NULL) {
+        syswarn("%s cannot fopen %s", sp->Name, sp->Filename);
+	if ((sp->F = fopen("/dev/null", "w")) == NULL)
 	    /* This really should not happen. */
-	    (void)fprintf(stderr, "buffchan %s cant fopen %s, %s\n",
-		    sp->Name, "/dev/null", strerror(errno));
-	    exit(1);
-	}
+            sysdie("%s cannot fopen /dev/null", sp->Name);
     }
     else if (fchmod((int)fileno(sp->F), 0444) < 0)
-	(void)fprintf(stderr, "buffchan %s cant fchmod %s %s\n",
-		sp->Name, sp->Filename, strerror(errno));
+        syswarn("%s cannot fchmod %s", sp->Name, sp->Filename);
 	
     if (BufferMode != '\0')
 	setbuf(sp->F, sp->Buffer);
@@ -161,34 +141,31 @@ STATIC void SITEopen(SITE *sp)
     /* Reset all counters. */
     sp->FlushLines = 0;
     sp->CloseLines = 0;
-    sp->LastFlushed = Now.time;
-    sp->LastClosed = Now.time;
-    sp->Dropped = FALSE;
+    sp->LastFlushed = Now;
+    sp->LastClosed = Now;
+    sp->Dropped = false;
 }
 
 
 /*
 **  Find a site, possibly create if not found.
 */
-STATIC SITE *
-SITEfind(Name, CanCreate)
-    char		*Name;
-    BOOL		CanCreate;
+static SITE *
+SITEfind(char *Name, bool CanCreate)
 {
-    register char	*p;
-    register int	i;
+    char	*p;
+    int	i;
     unsigned int	j;
-    register SITE	*sp;
+    SITE	*sp;
     SITEHASH		*shp;
     char		c;
     char		buff[BUFSIZ];
 
     /* Look for site in the hash table. */
-    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     SITE_HASH(Name, p, j);
     shp = SITE_BUCKET(j);
     for (c = *Name, sp = shp->Sites, i = shp->Used; --i >= 0; sp++)
-	if (c == sp->Name[0] && caseEQ(Name, sp->Name))
+	if (c == sp->Name[0] && strcasecmp(Name, sp->Name) == 0)
 	    return sp;
     if (!CanCreate)
 	return NULL;
@@ -196,18 +173,18 @@ SITEfind(Name, CanCreate)
     /* Adding a new site -- grow hash bucket if we need to. */
     if (shp->Used == shp->Size - 1) {
 	shp->Size *= 2;
-	RENEW(shp->Sites, SITE, shp->Size);
+        shp->Sites = xrealloc(shp->Sites, shp->Size * sizeof(SITE));
     }
     sp = &shp->Sites[shp->Used++];
 
     /* Fill in the structure for the new site. */
-    sp->Name = COPY(Name);
-    (void)sprintf(buff, (STRING)Format, Map ? MAPname(Name) : sp->Name);
-    sp->Filename = COPY(buff);
+    sp->Name = xstrdup(Name);
+    snprintf(buff, sizeof(buff), Format, Map ? MAPname(Name) : sp->Name);
+    sp->Filename = xstrdup(buff);
     if (BufferMode == 'u')
 	sp->Buffer = NULL;
     else if (BufferMode == 'b')
-	sp->Buffer = NEW(char, BUFSIZ);
+	sp->Buffer = xmalloc(BUFSIZ);
     SITEopen(sp);
 
     return sp;
@@ -217,18 +194,16 @@ SITEfind(Name, CanCreate)
 /*
 **  Flush a site -- close and re-open the file.
 */
-STATIC void
-SITEflush(sp)
-    register SITE	*sp;
+static void
+SITEflush(SITE *sp)
 {
-    register FILE	*F;
+    FILE	*F;
 
     if ((F = sp->F) != NULL) {
 	if (fflush(F) == EOF || ferror(F)
 	 || fchmod((int)fileno(F), 0664) < 0
 	 || fclose(F) == EOF)
-	    (void)fprintf(stderr, "buffchan %s cant close %s, %s\n",
-		    sp->Name, sp->Filename, strerror(errno));
+            syswarn("%s cannot close %s", sp->Name, sp->Filename);
 	sp->F = NULL;
     }
     if (!sp->Dropped)
@@ -239,14 +214,14 @@ SITEflush(sp)
 /*
 **  Flush all open sites.
 */
-STATIC void
-SITEflushall()
+static void
+SITEflushall(void)
 {
-    register SITEHASH	*shp;
-    register SITE	*sp;
-    register int	i;
+    SITEHASH	*shp;
+    SITE	*sp;
+    int	i;
 
-    for (shp = SITEtable; shp < ENDOF(SITEtable); shp++)
+    for (shp = SITEtable; shp < ARRAY_END(SITEtable); shp++)
 	for (sp = shp->Sites, i = shp->Used; --i >= 0; sp++)
 	    SITEflush(sp);
 }
@@ -255,43 +230,37 @@ SITEflushall()
 /*
 **  Write data to a site.
 */
-STATIC void
-SITEwrite(name, text, len)
-    register char	*name;
-    register char	*text;
-    register int	len;
+static void
+SITEwrite(char *name, char *text, size_t len)
 {
-    register SITE	*sp;
+    SITE	*sp;
 
-    sp = SITEfind(name, TRUE);
+    sp = SITEfind(name, true);
     if (sp->F == NULL)
 	SITEopen(sp);
 
-    if (fwrite((POINTER)text, (SIZE_T)1, (SIZE_T)len, sp->F) != len)
-	(void)fprintf(stderr, "buffchan %s cant write %s\n",
-		sp->Name, strerror(errno));
+    if (fwrite(text, 1, len, sp->F) != len)
+        syswarn("%s cannot write", sp->Name);
 
     /* Bump line count; see if time to close or flush. */
     if (CloseEvery && ++(sp->CloseLines) >= CloseEvery) {
 	SITEflush(sp);
 	return;
     }
-    if (CloseSeconds && sp->LastClosed + CloseSeconds < Now.time) {
+    if (CloseSeconds && sp->LastClosed + CloseSeconds < Now) {
 	SITEflush(sp);
 	return;
     }
     if (FlushEvery && ++(sp->FlushLines) >= FlushEvery) {
 	if (fflush(sp->F) == EOF || ferror(sp->F))
-	    (void)fprintf(stderr, "buffchan %s cant flush %s, %s\n",
-		    sp->Name, sp->Filename, strerror(errno));
-	sp->LastFlushed = Now.time;
+            syswarn("%s cannot flush %s", sp->Name, sp->Filename);
+	sp->LastFlushed = Now;
 	sp->FlushLines = 0;
     }
-    else if (FlushSeconds && sp->LastFlushed + FlushSeconds < Now.time) {
+    else if (FlushSeconds && sp->LastFlushed + FlushSeconds < Now) {
 	if (fflush(sp->F) == EOF || ferror(sp->F))
-	    (void)fprintf(stderr, "buffchan %s cant flush %s, %s\n",
-		    sp->Name, sp->Filename, strerror(errno));
-	sp->LastFlushed = Now.time;
+            syswarn("%s cannot flush %s", sp->Name, sp->Filename);
+	sp->LastFlushed = Now;
 	sp->FlushLines = 0;
     }
 }
@@ -300,118 +269,103 @@ SITEwrite(name, text, len)
 /*
 **  Handle a command message.
 */
-STATIC void
-Process(p)
-    register char	*p;
+static void
+Process(char *p)
 {
-    register SITE	*sp;
+    SITE	*sp;
 
-    if (*p == 'b' && EQn(p, "begin", 5))
+    if (*p == 'b' && strncmp(p, "begin", 5) == 0)
 	/* No-op. */
 	return;
 
-    if (*p == 'f' && EQn(p, "flush", 5)) {
+    if (*p == 'f' && strncmp(p, "flush", 5) == 0) {
 	for (p += 5; ISWHITE(*p); p++)
 	    continue;
 	if (*p == '\0')
 	    SITEflushall();
-	else if ((sp = SITEfind(p, FALSE)) != NULL)
+	else if ((sp = SITEfind(p, false)) != NULL)
 	    SITEflush(sp);
-	else
-	    /*(void)fprintf(stderr, "buffchan flush %s unknown site\n", p);*/
-	    ;
+	/*else
+	    fprintf(stderr, "buffchan flush %s unknown site\n", p);*/
 	return;
     }
 
-    if (*p == 'd' && EQn(p, "drop", 4)) {
+    if (*p == 'd' && strncmp(p, "drop", 4) == 0) {
 	for (p += 4; ISWHITE(*p); p++)
 	    continue;
 	if (*p == '\0')
 	    SITEcloseall();
-	else if ((sp = SITEfind(p, FALSE)) == NULL)
-	    (void)fprintf(stderr, "buffchan drop %s unknown site\n", p);
+	else if ((sp = SITEfind(p, false)) == NULL)
+            warn("drop %s unknown site", p);
 	else {
 	    SITEclose(sp);
-	    sp->Dropped = TRUE;
+	    sp->Dropped = true;
 	}
 	return;
     }
 
-    if (*p == 'r' && EQn(p, "readmap", 7)) {
+    if (*p == 'r' && strncmp(p, "readmap", 7) == 0) {
 	MAPread(Map);
 	return;
     }
 
     /* Other command messages -- ignored. */
-    (void)fprintf(stderr, "buffchan unknown message %s\n", p);
-}
-
-
-/*
-**  Print usage message and exit.
-*/
-STATIC NORETURN
-Usage()
-{
-    (void)fprintf(stderr, "Usage error.\n");
-    exit(1);
+    warn("unknown message %s", p);
 }
 
 
 /*
 **  Mark that we got a signal; let two signals kill us.
 */
-STATIC SIGHANDLER
-CATCHinterrupt(s)
-    int		s;
+static void
+CATCHinterrupt(int s)
 {
-    GotInterrupt = TRUE;
-    (void)signal(s, SIG_DFL);
+    GotInterrupt = true;
+    xsignal(s, SIG_DFL);
 }
 
 
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(int ac, char *av[])
 {
-    register QIOSTATE	*qp;
-    register int	i;
-    register int	Fields;
-    register char	*p;
-    register char	*next;
-    register char	*line;
+    QIOSTATE	*qp;
+    int	i;
+    int	Fields;
+    char	*p;
+    char	*next;
+    char	*line;
     char		*Directory;
-    BOOL		Redirect;
+    bool		Redirect;
     FILE		*F;
     char		*ERRLOG;
 
-    /* First thing, set up logging and our identity. */
-    openlog("buffchan", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);           
+    /* First thing, set up our identity. */
+    message_program_name = "buffchan";
 
     /* Set defaults. */
-    if (ReadInnConf() < 0) exit(1);
-    ERRLOG = COPY(cpcatpath(innconf->pathlog, _PATH_ERRLOG));
+    if (!innconf_read(NULL))
+        exit(1);
+    ERRLOG = concatpath(innconf->pathlog, INN_PATH_ERRLOG);
     Directory = NULL;
     Fields = 1;
     Format = NULL;
-    Redirect = TRUE;
-    GotInterrupt = FALSE;
-    (void)umask(NEWSUMASK);
+    Redirect = true;
+    GotInterrupt = false;
+    umask(NEWSUMASK);
 
-    (void)signal(SIGHUP, CATCHinterrupt);
-    (void)signal(SIGINT, CATCHinterrupt);
-    (void)signal(SIGQUIT, CATCHinterrupt);
-    (void)signal(SIGPIPE, CATCHinterrupt);
-    (void)signal(SIGTERM, CATCHinterrupt);
-    (void)signal(SIGALRM, CATCHinterrupt);
+    xsignal(SIGHUP, CATCHinterrupt);
+    xsignal(SIGINT, CATCHinterrupt);
+    xsignal(SIGQUIT, CATCHinterrupt);
+    xsignal(SIGPIPE, CATCHinterrupt);
+    xsignal(SIGTERM, CATCHinterrupt);
+    xsignal(SIGALRM, CATCHinterrupt);
 
     /* Parse JCL. */
     while ((i = getopt(ac, av, "bc:C:d:f:l:L:m:p:rs:u")) != EOF)
 	switch (i) {
 	default:
-	    Usage();
-	    /* NOTREACHED */
+            die("usage error");
+            break;
 	case 'b':
 	case 'u':
 	    BufferMode = i;
@@ -425,7 +379,7 @@ main(ac, av)
 	case 'd':
 	    Directory = optarg;
 	    if (Format == NULL)
-		Format ="%s";
+		Format =xstrdup("%s");
 	    break;
 	case 'f':
 	    Fields = atoi(optarg);
@@ -441,20 +395,14 @@ main(ac, av)
 	    MAPread(Map);
 	    break;
 	case 'p':
-	    if ((F = fopen(optarg, "w")) == NULL) {
-		(void)fprintf(stderr, "buffchan cant fopen %s %s\n",
-			optarg, strerror(errno));
-		exit(1);
-	    }
-	    (void)fprintf(F, "%ld\n", (long)getpid());
-	    if (ferror(F) || fclose(F) == EOF) {
-		(void)fprintf(stderr, "buffchan cant fclose %s %s\n",
-			optarg, strerror(errno));
-		exit(1);
-	    }
+	    if ((F = fopen(optarg, "w")) == NULL)
+                sysdie("cannot fopen %s", optarg);
+	    fprintf(F, "%ld\n", (long)getpid());
+	    if (ferror(F) || fclose(F) == EOF)
+                sysdie("cannot fclose %s", optarg);
 	    break;
 	case 'r':
-	    Redirect = FALSE;
+	    Redirect = false;
 	    break;
 	case 's':
 	    Format = optarg;
@@ -463,33 +411,27 @@ main(ac, av)
     ac -= optind;
     av += optind;
     if (ac)
-	Usage();
+	die("usage error");
 
     /* Do some basic set-ups. */
     if (Redirect)
-	(void)freopen(ERRLOG, "a", stderr);
+	freopen(ERRLOG, "a", stderr);
     if (Format == NULL) {
-	Format = NEW(char, strlen(innconf->pathoutgoing) + 1 + 2 + 1);
-	(void)sprintf(Format, "%s/%%s", innconf->pathoutgoing);
-    }
-    if (Directory && chdir(Directory) < 0) {
-	(void)fprintf(stderr, "buffchan cant chdir %s %s\n",
-	    Directory, strerror(errno));
-	exit(1);
+        Format = concatpath(innconf->pathoutgoing, "%s");
     }
+    if (Directory && chdir(Directory) < 0)
+        sysdie("cannot chdir to %s", Directory);
     SITEsetup();
 
     /* Read input. */
     for (qp = QIOfdopen((int)fileno(stdin)); !GotInterrupt ; ) {
 	if ((line = QIOread(qp)) == NULL) {
 	    if (QIOerror(qp)) {
-		(void)fprintf(stderr, "buffchan cant read %s\n",
-			strerror(errno));
+                syswarn("cannot read");
 		break;
 	    }
 	    if (QIOtoolong(qp)) {
-		(void)fprintf(stderr, "buffchan long_line");
-		(void)QIOread(qp);
+                warn("long line");
 		continue;
 	    }
 
@@ -515,11 +457,8 @@ main(ac, av)
 	*p++ = '\n';
 	i = p - line;
 
-	if (GetTimeInfo(&Now) < 0) {
-	    (void)fprintf(stderr, "buffchan cant gettime %s\n",
-		    strerror(errno));
-	    break;
-	}
+        /* Update the current time. */
+        Now = time(NULL);
 
 	/* Rest of the line is space-separated list of filenames. */
 	for (; *p; p = next) {
