@@ -1,6 +1,6 @@
-/*  $Id: cvtbatch.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: cvtbatch.c 8788 2009-11-15 09:27:11Z iulius $
 **
-**  Read file list on standard input and spew out batchfiles.
+**  Read file list on standard input and spew out batch files.
 */
 
 #include "config.h"
@@ -27,7 +27,8 @@ main(int ac, char *av[]) {
     bool	Dirty;
     TOKEN	token;
     ARTHANDLE	*art;
-    int		len;
+    size_t	len;
+    time_t      arrived;
 
     /* First thing, set up our identity. */
     message_program_name = "cvtbatch";
@@ -42,9 +43,11 @@ main(int ac, char *av[]) {
             die("usage error");
             break;
 	case 'w':
+            free(format);
 	    for (p = format = optarg; *p; p++) {
 		switch (*p) {
 		case FEED_BYTESIZE:
+                case FEED_TIMERECEIVED:
 		case FEED_FULLNAME:
 		case FEED_MESSAGEID:
 		case FEED_NAME:
@@ -73,14 +76,16 @@ main(int ac, char *av[]) {
 	if (!IsToken(line))
 	    continue;
 	token = TextToToken(line);
-	if ((art = SMretrieve(token, RETR_HEAD)) == NULL)
+	if ((art = SMretrieve(token, RETR_ALL)) == NULL)
 	    continue;
-        text = wire_findheader(art->data, art->len, "Message-ID");
+        text = wire_findheader(art->data, art->len, "Message-ID", true);
 	if (text == NULL) {
 	    SMfreearticle(art);
 	    continue;
 	}
 	len = art->len;
+        arrived = art->arrived;
+
 	for (r = text; r < art->data + art->len; r++) {
 	    if (*r == '\r' || *r == '\n')
 		break;
@@ -102,7 +107,7 @@ main(int ac, char *av[]) {
 	    case FEED_BYTESIZE:
 		if (Dirty)
 		    putchar(' ');
-		printf("%d", len);
+		printf("%lu", (unsigned long)len);
 		break;
 	    case FEED_FULLNAME:
 	    case FEED_NAME:
@@ -115,6 +120,11 @@ main(int ac, char *av[]) {
 		    putchar(' ');
 		printf("%s", q);
 		break;
+            case FEED_TIMERECEIVED:
+                if (Dirty)
+                    putchar(' ');
+                printf("%lu", (unsigned long) arrived);
+                break;
 	    }
 	    Dirty = true;
 	}
