@@ -1,4 +1,4 @@
-/*  $Id: makehistory.c 8535 2009-06-23 18:09:56Z iulius $
+/*  $Id: makehistory.c 8894 2010-01-17 13:04:04Z iulius $
 **
 **  Rebuild history/overview databases.
 */
@@ -584,7 +584,7 @@ DoArt(ARTHANDLE *art)
 	}
     }
     for (fp = ARTfields, i = 0; i < ARTfieldsize; i++, fp++) {
-        fp->Header = wire_findheader(art->data, art->len, fp->Headername);
+        fp->Header = wire_findheader(art->data, art->len, fp->Headername, false);
 
         /* Someone managed to break their server so that they were appending
            multiple Xref headers, and INN had a bug where it wouldn't notice
@@ -600,7 +600,7 @@ DoArt(ARTHANDLE *art)
                     break;
                 next++;
                 left = art->len - (next - art->data);
-                next = wire_findheader(next, left, fp->Headername);
+                next = wire_findheader(next, left, fp->Headername, false);
                 if (next != NULL)
                     fp->Header = next;
             }
@@ -659,7 +659,7 @@ DoArt(ARTHANDLE *art)
     }
     if (Missfieldsize > 0) {
 	for (fp = Missfields, i = 0; i < Missfieldsize; i++, fp++) {
-            fp->Header = wire_findheader(art->data, art->len, fp->Headername);
+            fp->Header = wire_findheader(art->data, art->len, fp->Headername, false);
             if (fp->Header != NULL) {
 		fp->HasHeader = true;
                 p = wire_endheader(fp->Header, art->data + art->len - 1);
@@ -727,7 +727,7 @@ DoArt(ARTHANDLE *art)
     } else {
         buffer_set(&buffer, Datep->Header, Datep->HeaderLength);
         buffer_append(&buffer, NUL, 1);
-        Posted = parsedate_rfc2822_lax(buffer.data);
+        Posted = parsedate_rfc5322_lax(buffer.data);
         if (Posted == (time_t) -1)
 	    Posted = Arrived;
     }
@@ -735,7 +735,7 @@ DoArt(ARTHANDLE *art)
     if (Expp->HasHeader) {
         buffer_set(&buffer, Expp->Header, Expp->HeaderLength);
         buffer_append(&buffer, NUL, 1);
-        Expires = parsedate_rfc2822_lax(buffer.data);
+        Expires = parsedate_rfc5322_lax(buffer.data);
         if (Expires == (time_t) -1)
             Expires = 0;
     }
