@@ -1,4 +1,4 @@
-/*  $Id: prunehistory.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: prunehistory.c 8584 2009-08-20 21:54:07Z iulius $
 **
 **  Prune file names from history file.
 */
@@ -21,7 +21,7 @@
 static void
 Usage(void)
 {
-    fprintf(stderr, "Usage:  prunehistory [-p] [-f file] [input]\n");
+    fprintf(stderr, "Usage:  prunehistory [-p] [-f file]\n");
     exit(1);
 }
 
