@@ -1,4 +1,4 @@
-/*  $Id: inews.c 7069 2004-12-19 22:15:41Z rra $
+/*  $Id: inews.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -10,15 +10,15 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <grp.h>
 #include <pwd.h>
 #include <sys/stat.h>
 
 #include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "paths.h"
+#include "inn/newsuser.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 
 /* Signature handling.  The separator will be appended before the signature,
    and at most SIG_MAXLINES will be appended. */
@@ -110,7 +110,7 @@ static HEADER	Table[] = {
 
 #define HDR(_x)	(Table[(_x)].Value)
 
-
+
 
 /*
 **  Send the server a quit message, wait for a reply.
@@ -134,7 +134,7 @@ QuitServer(int x)
 	*p = '\0';
     if ((p = strchr(buff, '\n')) != NULL)
 	*p = '\0';
-    if (atoi(buff) != NNTP_GOODBYE_ACK_VAL)
+    if (atoi(buff) != NNTP_OK_QUIT)
         die("server did not reply to quit properly: %s", buff);
     fclose(FromServer);
     fclose(ToServer);
@@ -179,7 +179,7 @@ TrimSpaces(char *p)
 
     for (start = p; ISWHITE(*start); start++)
 	continue;
-    for (p = start + strlen(start); p > start && CTYPE(isspace, p[-1]); )
+    for (p = start + strlen(start); p > start && isspace((unsigned char) p[-1]); )
 	*--p = '\0';
     return start;
 }
@@ -213,7 +213,6 @@ StripOffHeaders(char *article)
     char	*q;
     HEADER	*hp;
     char	c;
-    int	i;
 
     /* Set up the other headers list. */
     OtherSize = HEADER_DELTA;
@@ -221,7 +220,7 @@ StripOffHeaders(char *article)
     OtherCount = 0;
 
     /* Scan through buffer, a header at a time. */
-    for (i = 0, p = article; ; i++) {
+    for (p = article; ; ) {
 
 	if ((q = strchr(p, ':')) == NULL)
             die("no colon in header line \"%.30s...\"", p);
@@ -239,7 +238,7 @@ StripOffHeaders(char *article)
 	}
 
 	/* See if it's a known header. */
-	c = CTYPE(islower, *p) ? toupper(*p) : *p;
+	c = islower((unsigned char) *p) ? toupper(*p) : *p;
 	for (hp = Table; hp < ARRAY_END(Table); hp++)
 	    if (c == hp->Name[0]
 	     && p[hp->Size] == ':'
@@ -255,10 +254,6 @@ StripOffHeaders(char *article)
 		break;
 	    }
 
-	/* Too many headers? */
-	if (++i > 5 * HEADER_DELTA)
-            die("more than %d lines of header", i);
-
 	/* No; add it to the set of other headers. */
 	if (hp == ARRAY_END(Table)) {
 	    if (OtherCount >= OtherSize - 1) {
@@ -277,7 +272,7 @@ StripOffHeaders(char *article)
     return p + 1;
 }
 
-
+
 
 /*
 **  See if the user is allowed to cancel the indicated message.  Assumes
@@ -295,7 +290,7 @@ CheckCancel(char *msgid, bool JustReturn)
     fprintf(ToServer, "head %s\r\n", msgid);
     SafeFlush(ToServer);
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || atoi(buff) != NNTP_HEAD_FOLLOWS_VAL) {
+     || atoi(buff) != NNTP_OK_HEAD) {
 	if (JustReturn)
 	    return;
         die("server has no such article");
@@ -310,9 +305,9 @@ CheckCancel(char *msgid, bool JustReturn)
 	    *p = '\0';
 	if (buff[0] == '.' && buff[1] == '\0')
 	    break;
-        if (strncmp(buff, "Sender:", 7) == 0)
+        if (strncasecmp(buff, "Sender:", 7) == 0)
             strlcpy(remotefrom, TrimSpaces(&buff[7]), SMBUF);
-        else if (remotefrom[0] == '\0' && strncmp(buff, "From:", 5) == 0)
+        else if (remotefrom[0] == '\0' && strncasecmp(buff, "From:", 5) == 0)
             strlcpy(remotefrom, TrimSpaces(&buff[5]), SMBUF);
     }
     if (remotefrom[0] == '\0') {
@@ -337,32 +332,42 @@ CheckCancel(char *msgid, bool JustReturn)
 **  See if the user is the news administrator.
 */
 static bool
-AnAdministrator(char *name, gid_t group)
+AnAdministrator(void)
 {
-    struct passwd	*pwp;
-    struct group	*grp;
-    char		**mem;
-    char		*p;
+    uid_t               news_uid;
+    gid_t               news_gid;
 
     if (Revoked)
 	return false;
 
     /* Find out who we are. */
-    if ((pwp = getpwnam(NEWSUSER)) == NULL)
-	/* Silent falure; clients might not have the group. */
-	return false;
-    if (getuid() == pwp->pw_uid)
-	return true;
+    if (get_news_uid_gid(&news_uid, &news_gid, false) != 0) {
+        /* Silent failure; clients might not have the group. */
+        return false;
+    }
+    if (getuid() == news_uid)
+        return true;
 
-    /* See if the we're in the right group. */
-    if ((grp = getgrnam(NEWSGRP)) == NULL || (mem = grp->gr_mem) == NULL)
-	/* Silent falure; clients might not have the group. */
-	return false;
-    if (group == grp->gr_gid)
-	return true;
-    while ((p = *mem++) != NULL)
-	if (strcmp(name, p) == 0)
-	    return true;
+    /* See if we are in the right group and examine process
+     * supplementary groups, rather than the group(5) file entry.
+     */
+    {
+        int ngroups = getgroups(0, 0);
+        gid_t *groups, *gp;
+        int rv;
+        int rest;
+
+        groups = (gid_t *) xmalloc(ngroups * sizeof(*groups));
+        if ((rv = getgroups(ngroups, groups)) < 0) {
+            /* Silent failure; client doesn't have the group. */
+            return false;
+        }
+        for (rest = ngroups, gp = groups; rest > 0; rest--, gp++) {
+            if (*gp == news_gid)
+                return true;
+        }
+    }
+    
     return false;
 }
 
@@ -371,12 +376,11 @@ AnAdministrator(char *name, gid_t group)
 **  Check the control message, and see if it's legit.
 */
 static void
-CheckControl(char *ctrl, struct passwd *pwp)
+CheckControl(char *ctrl)
 {
     char	*p;
     char	*q;
     char		save;
-    char		name[SMBUF];
 
     /* Snip off the first word. */
     for (p = ctrl; ISWHITE(*p); p++)
@@ -388,7 +392,7 @@ CheckControl(char *ctrl, struct passwd *pwp)
     save = *p;
     *p = '\0';
 
-    if (strcmp(ctrl, "cancel") == 0) {
+    if (strcasecmp(ctrl, "cancel") == 0) {
 	for (q = p + 1; ISWHITE(*q); q++)
 	    continue;
 	if (*q == '\0')
@@ -396,16 +400,15 @@ CheckControl(char *ctrl, struct passwd *pwp)
 	if (!Spooling)
 	    CheckCancel(q, false);
     }
-    else if (strcmp(ctrl, "checkgroups") == 0
-	  || strcmp(ctrl, "ihave")       == 0
-	  || strcmp(ctrl, "sendme")      == 0
-	  || strcmp(ctrl, "newgroup")    == 0
-	  || strcmp(ctrl, "rmgroup")     == 0
-	  || strcmp(ctrl, "sendsys")     == 0
-	  || strcmp(ctrl, "senduuname")  == 0
-	  || strcmp(ctrl, "version")     == 0) {
-	strlcpy(name, pwp->pw_name, SMBUF);
-	if (!AnAdministrator(name, pwp->pw_gid))
+    else if (strcasecmp(ctrl, "checkgroups") == 0
+	  || strcasecmp(ctrl, "ihave")       == 0
+	  || strcasecmp(ctrl, "sendme")      == 0
+	  || strcasecmp(ctrl, "newgroup")    == 0
+	  || strcasecmp(ctrl, "rmgroup")     == 0
+	  || strcasecmp(ctrl, "sendsys")     == 0
+	  || strcasecmp(ctrl, "senduuname")  == 0
+	  || strcasecmp(ctrl, "version")     == 0) {
+	if (!AnAdministrator())
             die("ask your news administrator to do the %s for you", ctrl);
     }
     else {
@@ -414,7 +417,7 @@ CheckControl(char *ctrl, struct passwd *pwp)
     *p = save;
 }
 
-
+
 
 /*
 **  Parse the GECOS field to get the user's full name.  This comes Sendmail's
@@ -455,8 +458,8 @@ FormatUserName(struct passwd *pwp, char *node)
     for (out = outbuff; *p && !GECOSTERM(*p) && left; p++) {
 	if (*p == '&') {
 	    strncpy(out, pwp->pw_name, left);
-	    if (CTYPE(islower, *out)
-	     && (out == outbuff || !CTYPE(isalpha, out[-1])))
+	    if (islower((unsigned char) *out)
+	     && (out == outbuff || !isalpha((unsigned char) out[-1])))
 		*out = toupper(*out);
 	    while (*out) {
 		out++;
@@ -465,7 +468,7 @@ FormatUserName(struct passwd *pwp, char *node)
 	}
 	else if (*p == '-'
 	      && p > pwp->pw_gecos
-              && (CTYPE(isdigit, p[-1]) || CTYPE(isspace, p[-1])
+              && (isdigit((unsigned char) p[-1]) || isspace((unsigned char) p[-1])
                   || p[-1] == ']')) {
 	    out = outbuff;
             left = SMBUF - 1;
@@ -518,7 +521,6 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
     static char		PATHFLUFF[] = PATHMASTER;
     HEADER              *hp;
     char                *p;
-    TIMEINFO		Now;
     char		buff[SMBUF];
     char		from[SMBUF];
 
@@ -559,14 +561,14 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
 
     /* Set Subject; Control overrides the subject. */
     if (HDR(_control)) {
-	CheckControl(HDR(_control), pwp);
+	CheckControl(HDR(_control));
     }
     else {
 	p = HDR(_subject);
 	if (p == NULL)
             die("required Subject header is missing or empty");
 	else if (HDR(_alsocontrol))
-	    CheckControl(HDR(_alsocontrol), pwp);
+	    CheckControl(HDR(_alsocontrol));
 #if	0
 	if (strncmp(p, "Re: ", 4) == 0 && HDR(_references) == NULL)
             die("article subject begins with \"Re: \" but has no references");
@@ -611,9 +613,7 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
     /* Followup-To; checked with Newsgroups. */
 
     /* Check Expires. */
-    if (GetTimeInfo(&Now) < 0)
-        sysdie("cannot get the time");
-    if (HDR(_expires) && parsedate(HDR(_expires), &Now) == -1)
+    if (HDR(_expires) && parsedate_rfc5322_lax(HDR(_expires)) == -1)
         die("cannot parse \"%s\" as an expiration date", HDR(_expires));
 
     /* References; left alone. */
@@ -751,7 +751,7 @@ CheckIncludedText(char *p, int lines)
         die("more included text than new text");
 }
 
-
+
 
 /*
 **  Read stdin into a string and return it.  Can't use ReadInDescriptor
@@ -784,7 +784,7 @@ ReadStdin(void)
     return article;
 }
 
-
+
 
 /*
 **  Offer the article to the server, return its reply.
@@ -923,7 +923,7 @@ main(int ac, char *av[])
 	case 'x':
             Exclusions = concat(optarg, "!", (char *) 0);
 	    break;
-	 case 'p':
+	case 'p':
 	    port = atoi(optarg);
 	    break;
 	/* Header lines that can be specified on the command line. */
@@ -963,13 +963,14 @@ main(int ac, char *av[])
         port = NNTP_PORT;
 
     /* Try to open a connection to the server. */
-    if (NNTPremoteopen(port, &FromServer, &ToServer, buff) < 0) {
+    if (NNTPremoteopen(port, &FromServer, &ToServer, buff, sizeof(buff)) < 0) {
 	Spooling = true;
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
 	if ((p = strchr(buff, '\r')) != NULL)
 	    *p = '\0';
-	strcpy(SpoolMessage, buff[0] ? buff : NOCONNECT);
+	strlcpy(SpoolMessage, buff[0] ? buff : NOCONNECT,
+                sizeof(SpoolMessage));
         deadfile = concatpath(pwp->pw_dir, "dead.article");
     }
     else {
@@ -986,11 +987,15 @@ main(int ac, char *av[])
 	SafeFlush(ToServer);
 	if (fgets(buff, HEADER_STRLEN, FromServer) == NULL)
             sysdie("cannot tell server we're reading");
-	if ((j = atoi(buff)) != NNTP_BAD_COMMAND_VAL)
+	if ((j = atoi(buff)) != NNTP_ERR_COMMAND)
 	    i = j;
 
-	if (i != NNTP_POSTOK_VAL)
-            die("you do not have permission to post");
+        if (i != NNTP_OK_BANNER_POST) {
+            /* We try to authenticate in case it is all the same possible
+             * to post. */
+            if (NNTPsendpassword((char *)NULL, FromServer, ToServer) < 0)
+                die("you do not have permission to post");
+        }
 	deadfile = NULL;
     }
 
@@ -1009,9 +1014,9 @@ main(int ac, char *av[])
 	SigLines = 0;
     ProcessHeaders(AddOrg, i + SigLines, pwp);
     Length = strlen(article);
-    if ((innconf->localmaxartsize > 0)
-	    && (Length > (size_t)innconf->localmaxartsize))
-        die("article is larger than local limit of %ld bytes",
+    if ((innconf->localmaxartsize != 0)
+	    && (Length > innconf->localmaxartsize))
+        die("article is larger than local limit of %lu bytes",
             innconf->localmaxartsize);
 
     /* Do final checks. */
@@ -1052,13 +1057,13 @@ main(int ac, char *av[])
 
     /* Article is prepared, offer it to the server. */
     i = OfferArticle(buff, false);
-    if (i == NNTP_AUTH_NEEDED_VAL) {
+    if (i == NNTP_FAIL_AUTH_NEEDED) {
 	/* Posting not allowed, try to authorize. */
 	if (NNTPsendpassword((char *)NULL, FromServer, ToServer) < 0)
             sysdie("authorization error");
 	i = OfferArticle(buff, true);
     }
-    if (i != NNTP_START_POST_VAL)
+    if (i != NNTP_CONT_POST)
         die("server doesn't want the article: %s", buff);
 
     /* Write the headers, a blank line, then the article. */
@@ -1078,7 +1083,7 @@ main(int ac, char *av[])
 	*p = '\0';
     if ((p = strchr(buff, '\n')) != NULL)
 	*p = '\0';
-    if (atoi(buff) != NNTP_POSTEDOK_VAL)
+    if (atoi(buff) != NNTP_OK_POST)
         die("cannot send article to server: %s", buff);
 
     /* Close up. */
