@@ -1,4 +1,4 @@
-/*  $Id: rnews.c 7703 2008-01-12 09:22:18Z iulius $
+/*  $Id: rnews.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  A front-end for InterNetNews.
 **
@@ -21,10 +21,10 @@
 #include "inn/libinn.h"
 #include "inn/messages.h"
 #include "inn/newsuser.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 #include "inn/storage.h"
 #include "inn/wire.h"
-#include "nntp.h"
 
 
 typedef struct _HEADER {
@@ -77,7 +77,7 @@ StartChild(int fd, const char *path, const char *argv[])
 
     /* Get a child. */
     for (i = 0; (pid = fork()) < 0; i++) {
-	if (i == innconf->maxforks) {
+	if (i == (long) innconf->maxforks) {
             syswarn("cannot fork %s, spooling", path);
 	    return -1;
 	}
@@ -228,7 +228,7 @@ Process(char *article, size_t artlen)
 
     /* Make sure that all the headers are there, note the ID. */
     for (hp = RequiredHeaders; hp < ARRAY_END(RequiredHeaders); hp++) {
-        p = wire_findheader(wirefmt, length, hp->Name);
+        p = wire_findheader(wirefmt, length, hp->Name, true);
         if (p == NULL) {
             free(wirefmt);
 	    Reject(article, artlen, "bad_article missing %s", hp->Name);
@@ -271,7 +271,7 @@ Process(char *article, size_t artlen)
 	return false;
     }
     REMclean(buff);
-    if (!CTYPE(isdigit, buff[0])) {
+    if (!isdigit((unsigned char) buff[0])) {
         free(wirefmt);
         notice("bad_reply after ihave %s", buff);
 	return false;
@@ -325,7 +325,7 @@ Process(char *article, size_t artlen)
 	return false;
     }
     REMclean(buff);
-    if (!CTYPE(isdigit, buff[0])) {
+    if (!isdigit((unsigned char) buff[0])) {
         notice("bad_reply after article %s", buff);
 	return false;
     }
@@ -621,7 +621,7 @@ Unspool(void)
     struct dirent       *ep;
     bool	ok;
     struct stat		Sb;
-    char		hostname[10];
+    char		hostname[SMBUF];
     int			fd;
     size_t		i;
     char                *uuhost;
@@ -917,11 +917,11 @@ int main(int ac, char *av[])
 
     /* Open the link to the server. */
     if (remoteServer != NULL) {
-	if (!OpenRemote(remoteServer,port,buff,sizeof(buff)))
+	if (!OpenRemote(remoteServer, port, buff, sizeof(buff)))
 		CantConnect(buff,mode,fd);
     } else if (innconf->nnrpdposthost != NULL) {
 	if (!OpenRemote(innconf->nnrpdposthost,
-                        (port != NNTP_PORT) ? port : innconf->nnrpdpostport,
+                        (port != NNTP_PORT) ? (unsigned) port : innconf->nnrpdpostport,
                         buff, sizeof(buff)))
 		CantConnect(buff, mode, fd);
     }
@@ -930,7 +930,7 @@ int main(int ac, char *av[])
 	    /* If server rejected us, no point in continuing. */
 	    if (buff[0])
 		CantConnect(buff, mode, fd);
-	    if (!OpenRemote(NULL, (port != NNTP_PORT) ? port : innconf->port,
+	    if (!OpenRemote(NULL, (port != NNTP_PORT) ? (unsigned) port : innconf->port,
                             buff, sizeof(buff)))
 			CantConnect(buff, mode, fd);
 	}
