@@ -1,4 +1,4 @@
-/*  $Id: cc.c 7748 2008-04-06 13:49:56Z iulius $
+/*  $Id: cc.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Routines for the control channel.
 **
@@ -22,7 +22,7 @@
 #include "inn/innconf.h"
 #include "inn/qio.h"
 #include "innd.h"
-#include "inndcomm.h"
+#include "inn/inndcomm.h"
 #include "innperl.h"
 
 /*
@@ -67,7 +67,6 @@ static const char *	CCtimer(char *av[]);
 static const char *	CCtrace(char *av[]);
 static const char *	CCxabort(char *av[]);
 static const char *	CCxexec(char *av[]);
-static const char *	CCfilter(char *av[]);
 static const char *	CCperl(char *av[]);
 static const char *	CCpython(char *av[]);
 static const char *	CClowmark(char *av[]);
@@ -84,7 +83,9 @@ static char		CCbigreason[] = "1 Reason too long";
 static char		CCnotrunning[] = "1 Must be running";
 static struct buffer	CCreply;
 static CHANNEL		*CCchan;
+#ifdef HAVE_UNIX_DOMAIN_SOCKETS
 static int		CCwriter;
+#endif
 static CCDISPATCH	CCcommands[] = {
     {	SC_ADDHIST,	5, CCaddhist	},
     {	SC_ALLOW,	1, CCallow	},
@@ -94,7 +95,6 @@ static CCDISPATCH	CCcommands[] = {
     {	SC_CHECKFILE,	0, CCcheckfile	},
     {	SC_DROP,	1, CCdrop	},
     {	SC_FEEDINFO,	1, CCfeedinfo	},
-    {	SC_FILTER,	1, CCfilter     },
     {	SC_PERL,	1, CCperl       },
     {	SC_PYTHON,	1, CCpython     },
     {	SC_FLUSH,	1, CCflush	},
@@ -126,8 +126,8 @@ static CCDISPATCH	CCcommands[] = {
     {	SC_XEXEC,	1, CCxexec	}
 };
 
-static RETSIGTYPE CCresetup(int unused);
-
+static void CCresetup(int unused);
+
 
 void
 CCcopyargv(char *av[])
@@ -264,7 +264,7 @@ CCaddhist(char *av[])
 	ok = InndHisWrite(msgid, Data.Arrived, Data.Posted,
 			  Data.Expires, &token);
     else {
-	/* Possible race condition, but documented in ctlinnd manpage. */
+	/* Possible race condition, but documented in ctlinnd man page. */
 	InndHisOpen();
 	ok = InndHisWrite(msgid, Data.Arrived, Data.Posted,
 			  Data.Expires, &token);
@@ -275,7 +275,7 @@ CCaddhist(char *av[])
 
 
 /*
-**  Do the work to allow foreign connectiosn.
+**  Do the work to allow foreign connections.
 */
 static const char *
 CCallow(char *av[])
@@ -407,7 +407,7 @@ CCchgroup(char *av[])
     Rest = av[1];
     if (Rest[0] != NF_FLAG_ALIAS) {
 	Rest[1] = '\0';
-	if (CTYPE(isupper, Rest[0]))
+	if (isupper((unsigned char) Rest[0]))
 	    Rest[0] = tolower(Rest[0]);
     }
     return CCdochange(ngp, Rest);
@@ -423,7 +423,7 @@ CCcancel(char *av[])
     ARTDATA	Data;
     const char *	p, *msgid;
 
-    Data.Posted = Data.Arrived = Now.time;
+    Data.Posted = Data.Arrived = Now.tv_sec;
     Data.Expires = 0;
     Data.Feedsite = "?";
     if ((p = CCgetid(av[0], &msgid)) != NULL)
@@ -440,7 +440,7 @@ CCcancel(char *av[])
 	    return "1 Server paused";
 	if (ThrottledbyIOError)
 	    return "1 Server throttled";
-	/* Possible race condition, but documented in ctlinnd manpage. */
+	/* Possible race condition, but documented in ctlinnd man page. */
 	InndHisOpen();
 	ARTcancel(&Data, msgid, true);
 	InndHisClose();
@@ -495,9 +495,7 @@ CCcheckfile(char *unused[])
   if (errors == 0)
     return NULL;
 
-  buffer_resize(&CCreply, SMBUF);
-  snprintf(CCreply.data, CCreply.size, "1 Found %d errors -- see syslog",
-           errors);
+  buffer_sprintf(&CCreply, false, "1 Found %d errors -- see syslog", errors);
   return CCreply.data;
 }
 
@@ -564,36 +562,9 @@ CCfeedinfo(char *av[])
 
 
 static const char *
-CCfilter(char *av[] UNUSED)
+CCperl(char *av[] UNUSED)
 {
-#if defined(DO_TCL)
-    char	*p;
-
-    switch (av[0][0]) {
-    default:
-	return "1 Bad flag";
-    case 'y':
-	if (TCLFilterActive)
-	    return "1 tcl filter already enabled";
-	TCLfilter(true);
-	break;
-    case 'n':
-	if (!TCLFilterActive)
-	    return "1 tcl filter already disabled";
-	TCLfilter(false);
-	break;
-    }
-    return NULL;
-#else /* defined(DO_TCL) */
-    return "1 TCL filtering support not compiled in";
-#endif /* defined(DO_TCL) */
-}
-
-
-static const char *
-CCperl(char *av[])
-{
-#if defined(DO_PERL)
+#ifdef DO_PERL
     switch (av[0][0]) {
     default:
 	return "1 Bad flag";
@@ -610,20 +581,20 @@ CCperl(char *av[])
 	break;
     }
     return NULL;
-#else /* defined(DO_PERL) */
+#else
     return "1 Perl filtering support not compiled in";
-#endif /* defined(DO_PERL) */
+#endif
 }
 
 
 static const char *
 CCpython(char *av[] UNUSED)
 {
-#if defined(DO_PYTHON)
+#ifdef DO_PYTHON
     return PYcontrol(av);
-#else /* defined(DO_PYTHON) */
+#else
     return "1 Python filtering support not compiled in";
-#endif /* defined(DO_PYTHON) */
+#endif
 }
 
 
@@ -738,7 +709,7 @@ CChangup(char *av[])
 
     /* Parse the argument, a channel number. */
     for (p = av[0], fd = 0; *p; p++) {
-	if (!CTYPE(isdigit, *p))
+	if (!isdigit((unsigned char) *p))
 	    return "1 Bad channel number";
 	fd = fd * 10 + *p - '0';
     }
@@ -765,129 +736,156 @@ CChangup(char *av[])
 }
 
 
+/*
+**  Read a file containing lines of the form "newsgroup lowmark" and reset
+**  the low article number for the specified groups.
+*/
+static const char *
+CClowmark(char *av[])
+{
+    long lo;
+    char *line, *cp;
+    const char  *ret = NULL;
+    QIOSTATE *qp;
+    NEWSGROUP *ngp;
+
+    if (Mode != OMrunning)
+	return CCnotrunning;
+    if (ICDneedsetup)
+	return "1 Must first reload newsfeeds";
+    if ((qp = QIOopen(av[0])) == NULL) {
+	syslog(L_ERROR, "%s cant open %s %m", LogName, av[0]);
+	return "1 Cannot read input file";
+    }
+    while ((line = QIOread(qp)) != NULL) {
+	if (QIOerror(qp))
+		break;
+	if (QIOtoolong(qp)) {
+	    ret = "1 Malformed input line (too long)";
+	    break;
+	}
+	while (ISWHITE(*line))
+	    line++;
+	for (cp = line; *cp && !ISWHITE(*cp); cp++)
+	    ;
+	if (*cp == '\0') {
+	    ret = "1 Malformed input line (only one field)";
+	    break;
+	}
+	*cp++ = '\0';
+	while (ISWHITE(*cp))
+	    cp++;
+	if (strspn(cp, "0123456789") != strlen(cp)) {
+	    ret = "1 Malformed input line (non-digit in low mark)";
+	    break;
+	}
+	if ((lo = atol(cp)) == 0 && (cp[0] != '0' || cp[1] != '\0')) {
+	    ret = "1 Malformed input line (bad low mark)";
+	    break;
+	}
+        if ((ngp = NGfind(line)) == NULL) {
+	    /* ret = CCnogroup; break; */
+	    continue;
+	}
+        if (!NGlowmark(ngp, lo)) {
+	    ret = "1 Cannot set low mark - see syslog";
+	    break;
+	}
+    }
+    if (ret == NULL && QIOerror(qp)) {
+	syslog(L_ERROR, "%s cant read %s %m", LogName, av[0]);
+	ret = "1 Error reading input file";
+    }
+    QIOclose(qp);
+    ICDwrite();
+    return ret;
+}
+
+
 /*
 **  Return our operating mode.
 */
 static const char *
 CCmode(char *unused[] UNUSED)
 {
-    char	*p;
-    int		i;
-    int		h;
-    char	buff[BUFSIZ];
-#if defined(DO_PERL)
-    char        *stats;
-#endif /* defined(DO_PERL) */
+    int count, index;
 
-    /* FIXME: We assume here that BUFSIZ is >= 512, and that none of
-     * ModeReason, RejectReason, Reservation, NNRPReason, or the Perl filter
-     * statistics are longer than MAX_REASON_LEN bytes (or actually, the
-     * average of their lengths is <= MAX_REASON_LEN).  If this is not true,
-     * the sprintf's/strcpy's below are likely to overflow buff with somewhat
-     * nasty consequences...
-     */
+#ifdef DO_PERL
+    char *stats;
+#endif
 
-    p = buff;
-    p += strlen(strcpy(buff, "0 Server "));
+    buffer_sprintf(&CCreply, false, "0 Server ");
 
     /* Server's mode. */
     switch (Mode) {
     default:
-	sprintf(p, "Unknown %d", Mode);
-	p += strlen(p);
+        buffer_sprintf(&CCreply, true, "Unknown %d\n", Mode);
 	break;
     case OMrunning:
-	p += strlen(strcpy(p, "running"));
+        buffer_sprintf(&CCreply, true, "running\n");
 	break;
     case OMpaused:
-	p += strlen(strcpy(p, "paused "));
-	p += strlen(strcpy(p, ModeReason));
+        buffer_sprintf(&CCreply, true, "paused %s\n", ModeReason);
 	break;
     case OMthrottled:
-	p += strlen(strcpy(p, "throttled "));
-	p += strlen(strcpy(p, ModeReason));
+        buffer_sprintf(&CCreply, true, "throttled %s\n", ModeReason);
 	break;
     }
-    *p++ = '\n';
-    if (RejectReason) {
-	p += strlen(strcpy(p, "Rejecting "));
-	p += strlen(strcpy(p, RejectReason));
-    }
+    if (RejectReason)
+        buffer_sprintf(&CCreply, true, "Rejecting %s\n", RejectReason);
     else
-	p += strlen(strcpy(p, "Allowing remote connections"));
+        buffer_sprintf(&CCreply, true, "Allowing remote connections\n");
 
     /* Server parameters. */
-    for (i = 0, h = 0; CHANiter(&h, CTnntp) != NULL; )
-	i++;
-    *p++ = '\n';
-    sprintf(p, "Parameters c %ld i %ld (%d) l %ld o %d t %ld H %d T %d X %d %s %s",
-                  innconf->artcutoff, innconf->maxconnections, i,
-                  innconf->maxartsize, MaxOutgoing, (long)TimeOut.tv_sec,
-                  RemoteLimit, RemoteTotal, (int) RemoteTimer,
+    for (count = 0, index = 0; CHANiter(&index, CTnntp) != NULL; )
+	count++;
+    buffer_sprintf(&CCreply, true, "Parameters c %lu i %lu (%d) l %lu o %d"
+                   " t %ld H %d T %d X %ld %s %s\n",
+                  innconf->artcutoff, innconf->maxconnections, count,
+                  innconf->maxartsize, MaxOutgoing, (long) TimeOut.tv_sec,
+                  RemoteLimit, RemoteTotal, (long) RemoteTimer,
                   innconf->xrefslave ? "slave" : "normal",
                   AnyIncoming ? "any" : "specified");
-    p += strlen(p);
 
     /* Reservation. */
-    *p++ = '\n';
-    if (Reservation) {
-	sprintf(p, "Reserved %s", Reservation);
-	p += strlen(p);
-    }
+    if (Reservation)
+        buffer_sprintf(&CCreply, true, "Reserved %s\n", Reservation);
     else
-	p += strlen(strcpy(p, "Not reserved"));
+        buffer_sprintf(&CCreply, true, "Not reserved\n");
 
     /* Newsreaders. */
-    *p++ = '\n';
-    p += strlen(strcpy(p, "Readers "));
+    buffer_sprintf(&CCreply, true, "Readers ");
     if (innconf->readerswhenstopped)
-	p += strlen(strcpy(p, "independent "));
+        buffer_sprintf(&CCreply, true, "independent ");
     else
-	p += strlen(strcpy(p, "follow "));
+        buffer_sprintf(&CCreply, true, "follow ");
     if (NNRPReason == NULL)
-	p += strlen(strcpy(p, "enabled"));
-    else {
-	sprintf(p, "disabled %s", NNRPReason);
-	p += strlen(p);
-    }
-
-#if defined(DO_TCL)
-    *p++ = '\n';
-    p += strlen(strcpy(p, "Tcl filtering "));
-    if (TCLFilterActive)
-	p += strlen(strcpy(p, "enabled"));
+        buffer_sprintf(&CCreply, true, "enabled");
     else
-	p += strlen(strcpy(p, "disabled"));
-#endif /* defined(DO_TCL) */
+        buffer_sprintf(&CCreply, true, "disabled %s", NNRPReason);
 
-#if defined(DO_PERL)
-    *p++ = '\n';
-    p += strlen(strcpy(p, "Perl filtering "));
+#ifdef DO_PERL
+    buffer_sprintf(&CCreply, true, "\nPerl filtering ");
     if (PerlFilterActive)
-        p += strlen(strcpy(p, "enabled"));
+        buffer_sprintf(&CCreply, true, "enabled");
     else
-        p += strlen(strcpy(p, "disabled"));
-
-    /* Perl filter status. */
+        buffer_sprintf(&CCreply, true, "disabled");
     stats = PLstats();
     if (stats != NULL) {
-        *p++ = '\n';
-        p += strlen(strcpy(p, "Perl filter stats: "));
-        p += strlen(strcpy(p, stats));
+        buffer_sprintf(&CCreply, true, "\nPerl filter stats: %s", stats);
         free(stats);
     }    
-#endif /* defined(DO_PERL) */
+#endif
 
-#if defined(DO_PYTHON)
-    *p++ = '\n';
-    p += strlen(strcpy(p, "Python filtering "));
+#ifdef DO_PYTHON
+    buffer_sprintf(&CCreply, true, "\nPython filtering ");
     if (PythonFilterActive)
-        p += strlen(strcpy(p, "enabled"));
+        buffer_sprintf(&CCreply, true, "enabled");
     else
-        p += strlen(strcpy(p, "disabled"));
-#endif /* defined(DO_PYTHON) */
+        buffer_sprintf(&CCreply, true, "disabled");
+#endif
 
-    buffer_set(&CCreply, buff, strlen(buff) + 1);
+    buffer_append(&CCreply, "", 1);
     return CCreply.data;
 }
 
@@ -910,19 +908,15 @@ CClogmode(char *unused[])
 static const char *
 CCname(char *av[])
 {
-    static char		NL[] = "\n";
-    static char		NIL[] = "\0";
-    char		buff[SMBUF];
-    CHANNEL		*cp;
-    char		*p;
-    int			count;
-    int			i;
+    CHANNEL *cp;
+    int i, count;
+    const char *mode;
 
-    p = av[0];
-    if (*p != '\0') {
-	if ((cp = CHANfromdescriptor(atoi(p))) == NULL)
+    if (av[0][0] != '\0') {
+        cp = CHANfromdescriptor(atoi(av[0]));
+        if (cp == NULL)
 	    return xstrdup(CCnochannel);
-	snprintf(CCreply.data, CCreply.size, "0 %s", CHANname(cp));
+        buffer_sprintf(&CCreply, false, "0 %s", CHANname(cp));
 	return CCreply.data;
     }
     buffer_set(&CCreply, "0 ", 2);
@@ -930,45 +924,42 @@ CCname(char *av[])
 	if (cp->Type == CTfree)
 	    continue;
 	if (++count > 1)
-	    buffer_append(&CCreply, NL, 1);
-	p = CHANname(cp);
-	buffer_append(&CCreply, p, strlen(p));
+	    buffer_append(&CCreply, "\n", 1);
+        buffer_sprintf(&CCreply, true, "%s", CHANname(cp));
 	switch (cp->Type) {
 	case CTremconn:
-	    sprintf(buff, ":remconn::");
+            buffer_sprintf(&CCreply, true, ":remconn::");
 	    break;
 	case CTreject:
-	    sprintf(buff, ":reject::");
+            buffer_sprintf(&CCreply, true, ":reject::");
 	    break;
 	case CTnntp:
-           sprintf(buff, ":%s:%ld:%s",
-                    cp->State == CScancel ? "cancel" : "nntp",
-                    (long) Now.time - cp->LastActive,
-                    (cp->MaxCnx > 0 && cp->ActiveCnx == 0) ? "paused" : "");
+            mode = (cp->MaxCnx > 0 && cp->ActiveCnx == 0) ? "paused" : "";
+            buffer_sprintf(&CCreply, true, ":%s:%ld:%s",
+                           cp->State == CScancel ? "cancel" : "nntp",
+                           (long) Now.tv_sec - cp->LastActive, mode);
 	    break;
 	case CTlocalconn:
-	    sprintf(buff, ":localconn::");
+            buffer_sprintf(&CCreply, true, ":localconn::");
 	    break;
 	case CTcontrol:
-	    sprintf(buff, ":control::");
+            buffer_sprintf(&CCreply, true, ":control::");
 	    break;
 	case CTfile:
-	    sprintf(buff, "::");
+            buffer_sprintf(&CCreply, true, "::");
 	    break;
 	case CTexploder:
-	    sprintf(buff, ":exploder::");
+            buffer_sprintf(&CCreply, true, ":exploder::");
 	    break;
 	case CTprocess:
-	    sprintf(buff, ":");
+            buffer_sprintf(&CCreply, true, ":");
 	    break;
 	default:
-	    sprintf(buff, ":unknown::");
+            buffer_sprintf(&CCreply, true, ":unknown::");
 	    break;
 	}
-	p = buff;
-	buffer_append(&CCreply, p, strlen(p));
     }
-    buffer_append(&CCreply, NIL, 1);
+    buffer_append(&CCreply, "", 1);
     return CCreply.data;
 }
 
@@ -987,12 +978,11 @@ CCnewgroup(char *av[])
     char		*Name;
     char		*Rest;
     const char *		who;
-    char		*buff;
+    char		*buff = NULL;
     int			oerrno;
-    size_t              length;
 
     if (TIMES == NULL)
-	TIMES = concatpath(innconf->pathdb, _PATH_ACTIVETIMES);
+	TIMES = concatpath(innconf->pathdb, INN_PATH_ACTIVETIMES);
 
     Name = av[0];
     if (Name[0] == '.' || strspn(Name, "0123456789") == strlen(Name))
@@ -1008,9 +998,16 @@ CCnewgroup(char *av[])
     Rest = av[1];
     if (Rest[0] != NF_FLAG_ALIAS) {
 	Rest[1] = '\0';
-	if (CTYPE(isupper, Rest[0]))
+	if (isupper((unsigned char) Rest[0]))
 	    Rest[0] = tolower(Rest[0]);
     }
+
+    who = av[2];
+    if (*who == '\0')
+        who = NEWSMASTER;
+    if (!is_valid_utf8(who))
+        return "1 Invalid UTF-8 creator's name";
+
     if (strlen(Name) + strlen(Rest) > SMBUF - 24)
 	return "1 Name too long";
 
@@ -1028,13 +1025,7 @@ CCnewgroup(char *av[])
 	IOError(WHEN, oerrno);
     }
     else {
-	who = av[2];
-	if (*who == '\0')
-	    who = NEWSMASTER;
-
-        length = snprintf(NULL, 0, "%s %ld %s\n", Name, (long) Now.time, who) + 1;
-        buff = xmalloc(length);
-        snprintf(buff, length, "%s %ld %s\n", Name, (long) Now.time, who);
+        xasprintf(&buff, "%s %ld %s\n", Name, (long) Now.tv_sec, who);
 	if (xwrite(fd, buff, strlen(buff)) < 0) {
 	    oerrno = errno;
 	    syslog(L_ERROR, "%s cant write %s %m", LogName, TIMES);
@@ -1099,16 +1090,16 @@ CCparam(char *av[])
 	    return BADVAL;
 	break;
     case 'c':
-	innconf->artcutoff = atoi(p);
-	syslog(L_NOTICE, "%s changed -c %ld", LogName, innconf->artcutoff);
+	innconf->artcutoff = strtoul(p, NULL, 10);
+	syslog(L_NOTICE, "%s changed -c %lu", LogName, innconf->artcutoff);
 	break;
     case 'i':
-	innconf->maxconnections = atoi(p);
-	syslog(L_NOTICE, "%s changed -i %ld", LogName, innconf->maxconnections);
+	innconf->maxconnections = strtoul(p, NULL, 10);
+	syslog(L_NOTICE, "%s changed -i %lu", LogName, innconf->maxconnections);
 	break;
     case 'l':
-	innconf->maxartsize = atol(p);
-	syslog(L_NOTICE, "%s changed -l %ld", LogName, innconf->maxartsize);
+	innconf->maxartsize = strtoul(p, NULL, 10);
+	syslog(L_NOTICE, "%s changed -l %lu", LogName, innconf->maxartsize);
 	break;
     case 'n':
 	if (!CCparsebool('n', (bool *)&innconf->readerswhenstopped, *p))
@@ -1158,9 +1149,12 @@ CCblock(OPERATINGMODE NewMode, char *reason)
     if (*reason == '\0')
 	return CCnoreason;
 
-    if (strlen(reason) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe */
+    if (strlen(reason) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe. */
 	return CCbigreason;
 
+    if (!is_valid_utf8(reason))
+        return "1 Invalid UTF-8 reason";
+
     if (Reservation) {
 	if (strcmp(reason, Reservation) != 0) {
 	    snprintf(CCreply.data, CCreply.size, "1 Reserved \"%s\"",
@@ -1171,12 +1165,12 @@ CCblock(OPERATINGMODE NewMode, char *reason)
 	Reservation = NULL;
     }
 
-#if defined(DO_PERL)
+#ifdef DO_PERL
     PLmode(Mode, NewMode, reason);
-#endif /* defined(DO_PERL) */
-#if defined(DO_PYTHON)
+#endif
+#ifdef DO_PYTHON
     PYmode(Mode, NewMode, reason);
-#endif /* defined(DO_PYTHON) */
+#endif
 
     ICDwrite();
     InndHisClose();
@@ -1208,6 +1202,8 @@ CCpause(char *av[])
 	return "1 Already paused";
     case OMthrottled:
 	return "1 Already throttled";
+    case OMshutdown:
+        return "1 Shutting down";
     }
     return "1 Unknown mode";
 }
@@ -1239,8 +1235,10 @@ CCreaders(char *av[])
 	p = av[1];
 	if (*p == '\0')
 	    return CCnoreason;
-	if (strlen(p) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe */
+	if (strlen(p) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe. */
 	    return CCbigreason;
+        if (!is_valid_utf8(p))
+            return "1 Invalid UTF-8 reason";
 	NNRPReason = xstrdup(p);
 	break;
     }
@@ -1254,21 +1252,27 @@ CCreaders(char *av[])
 static const char *
 CCxexec(char *av[])
 {
-    char	*inndstart;
+    char	*innd;
     char	*p;
     int		i;
 
     if (CCargv == NULL)
 	return "1 no argv!";
     
-    inndstart = concatpath(innconf->pathbin, "inndstart");
+    innd = concatpath(innconf->pathbin, "innd");
     /* Get the pathname. */
     p = av[0];
-    if (*p == '\0' || strcmp(p, "innd") == 0 || strcmp(p, "inndstart") == 0)
-	CCargv[0] = inndstart;
+    if (*p == '\0' || strcmp(p, "innd") == 0)
+	CCargv[0] = innd;
     else
 	return "1 Bad value";
 
+#ifdef DO_PERL
+    PLmode(Mode, OMshutdown, av[0]);
+#endif
+#ifdef DO_PYTHON
+    PYmode(Mode, OMshutdown, av[0]);
+#endif
     JustCleanup();
     syslog(L_NOTICE, "%s execv %s", LogName, CCargv[0]);
 
@@ -1291,8 +1295,10 @@ CCreject(char *av[])
 {
     if (RejectReason)
 	return "1 Already rejecting";
-    if (strlen(av[0]) > MAX_REASON_LEN)	/* MAX_REASON_LEN is as big as is safe */
+    if (strlen(av[0]) > MAX_REASON_LEN)	/* MAX_REASON_LEN is as big as is safe. */
 	return CCbigreason;
+    if (!is_valid_utf8(av[0]))
+        return "1 Invalid UTF-8 reason";
     RejectReason = xstrdup(av[0]);
     return NULL;
 }
@@ -1304,15 +1310,16 @@ CCreject(char *av[])
 static const char *
 CCreload(char *av[])
 {
-    static char	BADSCHEMA[] = "1 Can't read schema";
-#if defined(DO_PERL)
-    static char BADPERLRELOAD[] = "1 Failed to define filter_art" ;
-#endif /* defined(DO_PERL) */
-#if defined(DO_PYTHON)
-    static char BADPYRELOAD[] = "1 Failed to reload filter_innd.py" ;
-#endif /* defined(DO_PYTHON) */
     const char *p;
+
+#ifdef DO_PERL
+    static char BADPERLRELOAD[] = "1 Failed to define filter_art" ;
     char *path;
+#endif
+
+#ifdef DO_PYTHON
+    static char BADPYRELOAD[] = "1 Failed to reload filter_innd.py" ;
+#endif
 
     p = av[0];
     if (*p == '\0' || strcmp(p, "all") == 0) {
@@ -1324,21 +1331,16 @@ CCreload(char *av[])
 	    InndHisOpen();
 	ICDwrite();
 	ICDsetup(true);
-	if (!ARTreadschema())
-	    return BADSCHEMA;
-#if defined(DO_TCL)
-	TCLreadfilter();
-#endif /* defined(DO_TCL) */
-#if defined(DO_PERL)
-        path = concatpath(innconf->pathfilter, _PATH_PERL_FILTER_INND);
+#ifdef DO_PERL
+        path = concatpath(innconf->pathfilter, INN_PATH_PERL_FILTER_INND);
         PERLreadfilter(path, "filter_art") ;
         free(path);
-#endif /* defined(DO_PERL) */
-#if defined(DO_PYTHON)
+#endif
+#ifdef DO_PYTHON
 	syslog(L_NOTICE, "reloading pyfilter");
-	 PYreadfilter();
+        PYreadfilter();
 	syslog(L_NOTICE, "reloaded pyfilter OK");
-#endif /* DO_PYTHON */
+#endif
 	p = "all";
     }
     else if (strcmp(p, "active") == 0 || strcmp(p, "newsfeeds") == 0) {
@@ -1352,11 +1354,8 @@ CCreload(char *av[])
 	InndHisClose();
 	InndHisOpen();
     }
-    else if (strcmp(p, "incoming.conf") == 0)
-	RCreadlist();
-    else if (strcmp(p, "overview.fmt") == 0) {
-	if (!ARTreadschema())
-	    return BADSCHEMA;
+    else if (strcmp(p, "incoming.conf") == 0) {
+        RCreadlist();
     }
 #if 0 /* we should check almost all innconf parameter, but the code
          is still incomplete for innd, so just commented out */
@@ -1388,7 +1387,7 @@ CCreload(char *av[])
 	    Pathalias.Used = strlen(innconf->pathalias) + 1;
 	    Pathalias.Data = xmalloc(Pathalias.Used + 1);
 	    sprintf(Pathalias.Data, "%s!", innconf->pathalias);
-        }
+	}
         if (Pathcluster.Used > 0)
             free(Pathcluster.Data);
         if (innconf->pathcluster == NULL) {
@@ -1398,27 +1397,22 @@ CCreload(char *av[])
             Pathcluster.Used = strlen(innconf->pathcluster) + 1;
             Pathcluster.Data = xmalloc(Pathcluster.Used + 1);
             sprintf(Pathcluster.Data, "%s!", innconf->pathcluster);
-	}
+        }
     }
 #endif
-#if defined(DO_TCL)
-    else if (strcmp(p, "filter.tcl") == 0) {
-	TCLreadfilter();
-    }
-#endif /* defined(DO_TCL) */
-#if defined(DO_PERL)
+#ifdef DO_PERL
     else if (strcmp(p, "filter.perl") == 0) {
-        path = concatpath(innconf->pathfilter, _PATH_PERL_FILTER_INND);
+        path = concatpath(innconf->pathfilter, INN_PATH_PERL_FILTER_INND);
         if (!PERLreadfilter(path, "filter_art"))
             return BADPERLRELOAD;
     }
-#endif /* defined(DO_PERL) */
-#if defined(DO_PYTHON)
+#endif
+#ifdef DO_PYTHON
     else if (strcmp(p, "filter.python") == 0) {
 	if (!PYreadfilter())
 	    return BADPYRELOAD;
     }
-#endif /* defined(DO_PYTHON) */
+#endif
     else
 	return "1 Unknown reload type";
 
@@ -1469,8 +1463,10 @@ CCreserve(char *av[])
 	/* Trying to make a reservation. */
 	if (Reservation)
 	    return "1 Already reserved";
-	if (strlen(p) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe */
+	if (strlen(p) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe. */
 	    return CCbigreason;
+        if (!is_valid_utf8(p))
+            return "1 Invalid UTF-8 reason";
 	Reservation = xstrdup(p);
     }
     else {
@@ -1529,6 +1525,12 @@ CCsend(char *av[])
 static const char *
 CCshutdown(char *av[])
 {
+#ifdef DO_PERL
+    PLmode(Mode, OMshutdown, av[0]);
+#endif
+#ifdef DO_PYTHON
+    PYmode(Mode, OMshutdown, av[0]);
+#endif
     syslog(L_NOTICE, "%s shutdown %s", LogName, av[0]);
     CleanupAndExit(0, av[0]);
     /* NOTREACHED */
@@ -1574,8 +1576,8 @@ CCsignal(char *av[])
 	oerrno = errno;
 	syslog(L_ERROR, "%s cant kill %ld %d site %s, %m", LogName, 
 		(long) sp->pid, s, p);
-	snprintf(CCreply.data, CCreply.size, "1 Can't signal process %ld, %s",
-		(long) sp->pid, strerror(oerrno));
+        buffer_sprintf(&CCreply, false, "1 Can't signal process %ld: %s",
+                       (long) sp->pid, strerror(oerrno));
 	return CCreply.data;
     }
 
@@ -1601,6 +1603,8 @@ CCthrottle(char *av[])
 	return CCblock(OMthrottled, p);
     case OMthrottled:
 	return "1 Already throttled";
+    case OMshutdown:
+        return "1 Shutting down";
     }
     return "1 unknown mode";
 }
@@ -1611,20 +1615,20 @@ CCthrottle(char *av[])
 static const char *
 CCtimer(char *av[])
 {
-    int                 value;
+    unsigned long    value;
     char                *p;
     
     if (strcmp(av[0], "off") == 0)
 	value = 0;
     else {
 	for (p = av[0]; *p; p++) {
-	    if (!CTYPE(isdigit, *p))
+	    if (!isdigit((unsigned char) *p))
 		return "1 parameter should be a number or 'off'";
 	}
-	value = atoi(av[0]);
+	value = strtoul(av[0], NULL, 10);
     }
     innconf->timer = value;
-    if (innconf->timer)
+    if (innconf->timer != 0)
         TMRinit(TMR_MAX);
     else
 	TMRinit(0);
@@ -1650,17 +1654,17 @@ CCstathist(char *av[])
 static const char *
 CCstatus(char *av[])
 {
-    int                 value;
+    unsigned long       value;
     char                *p;
     
     if (strcmp(av[0], "off") == 0)
 	value = 0;
     else {
 	for (p = av[0]; *p; p++) {
-	    if (!CTYPE(isdigit, *p))
+	    if (!isdigit((unsigned char) *p))
 		return "1 parameter should be a number or 'off'";
 	}
-	value = atoi(av[0]);
+	value = strtoul(av[0], NULL, 10);
     }
     innconf->status = value;
     return NULL;
@@ -1708,7 +1712,7 @@ CCtrace(char *av[])
     return NULL;
 }
 
-
+
 
 /*
 **  Split up the text into fields and stuff them in argv.  Return the
@@ -1737,24 +1741,25 @@ CCargsplit(char *p, char *end, char **argv, int size)
 static void
 CCreader(CHANNEL *cp)
 {
-    static char		TOOLONG[] = "0 Reply too long for server to send";
     CCDISPATCH		*dp;
     const char *	p;
-    char		*q;
     ICC_MSGLENTYPE	bufflen;
     ICC_PROTOCOLTYPE	protocol ;
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
-    struct sockaddr_un	client;
+    static char         TOOLONG[] = "0 Reply too long for server to send";
+    struct sockaddr_un  client;
 #else
-    int			written;
+    int                 written;
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
     int			i;
     char                buff[BIG_BUFFER + 2];
-    char                copy[BIG_BUFFER + 2];
     char		*argv[SC_MAXFIELDS + 2];
     int			argc;
     int			len;
     char		*tbuff ;
+    const char *start;
+    char *copy;
+    size_t offset;
 
     if (cp != CCchan) {
 	syslog(L_ERROR, "%s internal CCreader wrong channel 0x%p not 0x%p",
@@ -1778,16 +1783,16 @@ CCreader(CHANNEL *cp)
 
     memcpy (&protocol,buff,sizeof (protocol)) ;
     memcpy (&bufflen,buff + sizeof (protocol),sizeof (bufflen)) ;
-    bufflen = ntohs (bufflen) ;
+    bufflen = ntohs (bufflen);
     
     if (i != bufflen) {
 	syslog(L_ERROR, "%s cant recv CCreader short-read %m", LogName);
 	return;
     }
 
-    i -= HEADER_SIZE ;
-    memmove (buff,buff + HEADER_SIZE,i) ;
-    buff[i] = '\0';
+    bufflen -= HEADER_SIZE;
+    memmove(buff, buff + HEADER_SIZE, bufflen);
+    buff[bufflen] = '\0';
 
     if (protocol != ICC_PROTOCOL_1) {
         syslog(L_ERROR, "%s CCreader protocol mismatch", LogName) ;
@@ -1839,18 +1844,25 @@ CCreader(CHANNEL *cp)
 
 #endif /* defined (HAVE_UNIX_DOMAIN_SOCKETS) */
     
-    /* Copy to a printable buffer, and log. */
-    strcpy(copy, buff);
-    for (p = NULL, q = copy; *q; q++)
-	if (*q == SC_SEP) {
-	    *q = ':';
-	    if (p == NULL)
-		p = q + 1;
-	}
-    syslog(L_CC_CMD, "%s", p ? p : copy);
+    /* Copy to a printable buffer, and log.  We skip the first
+       SC_SEP-delimited field.  Note that the protocol allows for nuls in the
+       message, which we'll replace with ? for logging. */
+    copy = xmalloc(bufflen + 1);
+    memcpy(copy, buff, bufflen);
+    copy[bufflen] = '\0';
+    for (offset = 0, start = NULL; offset < (size_t) bufflen; offset++)
+        if (copy[offset] == SC_SEP) {
+            copy[offset] = ':';
+            if (start == NULL)
+                start = copy + offset + 1;
+        } else if (copy[offset] == '\0') {
+            copy[offset] = '?';
+        }
+    notice("ctlinnd command %s", start != NULL ? start : copy);
+    free(copy);
 
     /* Split up the fields, get the command letter. */
-    if ((argc = CCargsplit(buff, &buff[i], argv, ARRAY_SIZE(argv))) < 2
+    if ((argc = CCargsplit(buff, &buff[bufflen], argv, ARRAY_SIZE(argv))) < 2
      || argc == ARRAY_SIZE(argv)) {
 	syslog(L_ERROR, "%s bad_fields CCreader", LogName);
 	return;
@@ -1886,13 +1898,13 @@ CCreader(CHANNEL *cp)
     memcpy (tbuff,&bufflen,sizeof (bufflen)) ;
     tbuff += sizeof (bufflen) ;
 
-    strcpy (tbuff,p) ;
+    strlcpy(tbuff, p, len + 1 - sizeof(protocol) - sizeof(bufflen));
     tbuff -= HEADER_SIZE ;
 
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     memset(&client, 0, sizeof client);
     client.sun_family = AF_UNIX;
-    strcpy(client.sun_path, argv[0]);
+    strlcpy(client.sun_path, argv[0], sizeof(client.sun_path));
     if (sendto(CCwriter, tbuff, len, 0,
 	    (struct sockaddr *) &client, SUN_LEN(&client)) < 0) {
 	i = errno;
@@ -1906,11 +1918,12 @@ CCreader(CHANNEL *cp)
     if ((i = open(argv[0], O_WRONLY | O_NDELAY)) < 0)
 	syslog(L_ERROR, "%s cant open %s %m", LogName, argv[0]);
     else {
-	if ((written = write(i, tbuff, len)) != len)
+	if ((written = write(i, tbuff, len)) != len) {
 	    if (written < 0)
 		syslog(L_ERROR, "%s cant write %s %m", LogName, argv[0]);
-	    else
+            else
 		syslog(L_ERROR, "%s cant write %s", LogName, argv[0]);
+        }
 	if (close(i) < 0)
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, argv[0]);
     }
@@ -1943,7 +1956,7 @@ CCsetup(void)
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
     if (CCpath == NULL)
-	CCpath = concatpath(innconf->pathrun, _PATH_NEWSCONTROL);
+	CCpath = concatpath(innconf->pathrun, INN_PATH_NEWSCONTROL);
     /* Remove old detritus. */
     if (unlink(CCpath) < 0 && errno != ENOENT) {
 	syslog(L_FATAL, "%s cant unlink %s %m", LogName, CCpath);
@@ -1958,7 +1971,7 @@ CCsetup(void)
     }
     memset(&server, 0, sizeof server);
     server.sun_family = AF_UNIX;
-    strcpy(server.sun_path, CCpath);
+    strlcpy(server.sun_path, CCpath, sizeof(server.sun_path));
     if (bind(i, (struct sockaddr *) &server, SUN_LEN(&server)) < 0) {
 	syslog(L_FATAL, "%s cant bind %s %m", LogName, CCpath);
 	exit(1);
@@ -2028,7 +2041,7 @@ CCclose(void)
 /*
 **  Restablish the control channel.
 */
-static RETSIGTYPE
+static void
 CCresetup(int unused)
 {
 #ifndef HAVE_SIGACTION
@@ -2039,69 +2052,3 @@ CCresetup(int unused)
     CCclose();
     CCsetup();
 }
-
-
-/*
- * Read a file containing lines of the form "newsgroup lowmark",
- * and reset the low article number for the specified groups.
- */
-static const char *
-CClowmark(char *av[])
-{
-    long lo;
-    char *line, *cp;
-    const char  *ret = NULL;
-    QIOSTATE *qp;
-    NEWSGROUP *ngp;
-
-    if (Mode != OMrunning)
-	return CCnotrunning;
-    if (ICDneedsetup)
-	return "1 Must first reload newsfeeds";
-    if ((qp = QIOopen(av[0])) == NULL) {
-	syslog(L_ERROR, "%s cant open %s %m", LogName, av[0]);
-	return "1 Cannot read input file";
-    }
-    while ((line = QIOread(qp)) != NULL) {
-	if (QIOerror(qp))
-		break;
-	if (QIOtoolong(qp)) {
-	    ret = "1 Malformed input line (too long)";
-	    break;
-	}
-	while (ISWHITE(*line))
-	    line++;
-	for (cp = line; *cp && !ISWHITE(*cp); cp++)
-	    ;
-	if (*cp == '\0') {
-	    ret = "1 Malformed input line (only one field)";
-	    break;
-	}
-	*cp++ = '\0';
-	while (ISWHITE(*cp))
-	    cp++;
-	if (strspn(cp, "0123456789") != strlen(cp)) {
-	    ret = "1 Malformed input line (non-digit in low mark)";
-	    break;
-	}
-	if ((lo = atol(cp)) == 0 && (cp[0] != '0' || cp[1] != '\0')) {
-	    ret = "1 Malformed input line (bad low mark)";
-	    break;
-	}
-        if ((ngp = NGfind(line)) == NULL) {
-	    /* ret = CCnogroup; break; */
-	    continue;
-	}
-        if (!NGlowmark(ngp, lo)) {
-	    ret = "1 Cannot set low mark - see syslog";
-	    break;
-	}
-    }
-    if (ret == NULL && QIOerror(qp)) {
-	syslog(L_ERROR, "%s cant read %s %m", LogName, av[0]);
-	ret = "1 Error reading input file";
-    }
-    QIOclose(qp);
-    ICDwrite();
-    return ret;
-}
