@@ -1,4 +1,4 @@
-/*  $Id: perl.c 6242 2003-02-21 15:33:15Z alexk $
+/*  $Id: perl.c 8968 2010-02-08 21:06:43Z iulius $
 **
 **  Perl filtering support for innd.
 **
@@ -14,18 +14,10 @@
 **
 **  Future work:
 **
-**   - What we're doing with Path headers right now doesn't work for folded
-**     headers.  It's also kind of gross.  There has to be a better way of
-**     handling this.
-**
 **   - The breakdown between this file, lib/perl.c, and nnrpd/perl.c should
 **     be rethought, ideally in the light of supporting multiple filters in
 **     different languages.
 **
-**   - We're still calling strlen() on artBody, which should be avoidable
-**     since we've already walked it several times.  We should just cache
-**     the length somewhere for speed.
-**
 **   - Variable and key names should be standardized between this and nnrpd.
 **
 **   - The XS code is still calling CC* functions.  The common code between
@@ -43,6 +35,7 @@
 #if DO_PERL
 
 #include "clibrary.h"
+#include "inn/wire.h"
 #include "innd.h"
 
 #include <EXTERN.h>
@@ -52,8 +45,15 @@
 
 #include "innperl.h"
 
-/* From art.c.  Ew.  Need header parsing that doesn't use globals. */
-extern char             *filterPath;
+/* Prototypes for XS functions. */
+XS(XS_INN_addhist);
+XS(XS_INN_article);
+XS(XS_INN_cancel);
+XS(XS_INN_filesfor);
+XS(XS_INN_havehist);
+XS(XS_INN_head);
+XS(XS_INN_newsgroup);
+
 
 /*
 **  Run an incoming article through the Perl article filter.  Returns NULL
@@ -69,44 +69,38 @@ PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
     CV *        filter;
     int         i, rc;
     char *      p;
-    static SV * body = NULL;
     static char buf[256];
 
     if (!PerlFilterActive) return NULL;
     filter = perl_get_cv("filter_art", 0);
     if (!filter) return NULL;
 
-    /* Create %hdr and stash a copy of every known header.  Path has to be
-       handled separately since it's been munged by article processing. */
+    /* Create %hdr and stash a copy of every known header. */
     hdr = perl_get_hv("hdr", 1);
     for (i = 0 ; i < MAX_ARTHEADER ; i++) {
 	if (HDR_FOUND(i)) {
 	    hp = &ARTheaders[i];
-            hv_store(hdr, (char *) hp->Name, hp->Size, newSVpv(HDR(i), 0), 0);
+            (void) hv_store(hdr, (char *) hp->Name, hp->Size, newSVpv(HDR(i), 0), 0);
 	}
     }
 
     /* Store the article body.  We don't want to make another copy of it,
-       since it could potentially be quite large.  Instead, stash the
-       pointer in the static SV * body.  We set LEN to 0 and inc the
-       refcount to tell Perl not to free it (either one should be enough).
-       Requires 5.004.  In testing, this produced a 17% speed improvement
-       over making a copy of the article body for a fairly heavy filter. */
+       since it could potentially be quite large.  In testing, this produced
+       a 17% speed improvement over making a copy of the article body
+       for a fairly heavy filter.
+       Available since Perl 5.7.1, newSVpvn_share allows to avoid such
+       a copy (getting round its use for older versions of Perl leads
+       to unreliable SV * bodies as for regexps).  And for Perl not to
+       compute a hash for artBody, we give it "42". */
     if (artBody) {
-        if (!body) {
-            body = newSV(0);
-            (void) SvUPGRADE(body, SVt_PV);
-        }
-        SvPVX(body) = artBody;
-        SvCUR_set(body, artLen);
-        SvLEN_set(body, 0);
-        SvPOK_on(body);
-        (void) SvREADONLY_on(body);
-        (void) SvREFCNT_inc(body);
-        hv_store(hdr, "__BODY__", 8, body, 0);
+#if (PERL_REVISION == 5) && ((PERL_VERSION < 7) || ((PERL_VERSION == 7) && (PERL_SUBVERSION < 1)))
+        (void) hv_store(hdr, "__BODY__", 8, newSVpv(artBody, artLen), 0);
+#else
+        (void) hv_store(hdr, "__BODY__", 8, newSVpvn_share(artBody, artLen, 42), 0);
+#endif /* Perl < 5.7.1 */
     }
 
-    hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
+    (void) hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
 
     ENTER;
     SAVETMPS;
@@ -138,7 +132,8 @@ PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 
 
 /*
-**  Run an incoming message ID from CHECK or IHAVE through the Perl filter.
+**  Run an incoming message-ID from CHECK, IHAVE or TAKETHIS through
+**  the Perl filter.
 **  Returns NULL to accept the article or a rejection message to reject it.
 */
 char *
@@ -202,20 +197,24 @@ PLmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
     mode = perl_get_hv("mode", 1);
 
     if (Mode == OMrunning)
-        hv_store(mode, "Mode", 4, newSVpv("running", 0), 0);
+        (void) hv_store(mode, "Mode", 4, newSVpv("running", 7), 0);
     if (Mode == OMpaused)
-        hv_store(mode, "Mode", 4, newSVpv("paused", 0), 0);
+        (void) hv_store(mode, "Mode", 4, newSVpv("paused", 6), 0);
     if (Mode == OMthrottled)
-        hv_store(mode, "Mode", 4, newSVpv("throttled", 0), 0);
+        (void) hv_store(mode, "Mode", 4, newSVpv("throttled", 9), 0);
+    if (Mode == OMshutdown)
+        (void) hv_store(mode, "Mode", 4, newSVpv("shutdown", 8), 0);
 
     if (NewMode == OMrunning)
-        hv_store(mode, "NewMode", 7, newSVpv("running", 0), 0);
+        (void) hv_store(mode, "NewMode", 7, newSVpv("running", 7), 0);
     if (NewMode == OMpaused)
-        hv_store(mode, "NewMode", 7, newSVpv("paused", 0), 0);
+        (void) hv_store(mode, "NewMode", 7, newSVpv("paused", 6), 0);
     if (NewMode == OMthrottled)
-        hv_store(mode, "NewMode", 7, newSVpv("throttled", 0), 0);
+        (void) hv_store(mode, "NewMode", 7, newSVpv("throttled", 9), 0);
+    if (NewMode == OMshutdown)
+        (void) hv_store(mode, "NewMode", 7, newSVpv("shutdown", 8), 0);
 
-    hv_store(mode, "reason", 6, newSVpv(reason, 0), 0);
+    (void) hv_store(mode, "reason", 6, newSVpv(reason, 0), 0);
 
     PUSHMARK(SP);
     perl_call_sv((SV *) filter, G_EVAL|G_DISCARD|G_NOARGS);
@@ -240,6 +239,7 @@ char *
 PLstats(void)
 {
     dSP;
+    char *argv[] = { NULL };
     
     if (perl_get_cv("filter_stats", false) == NULL)
         return NULL;
@@ -249,7 +249,7 @@ PLstats(void)
 
 	ENTER;
 	SAVETMPS;
-	perl_call_argv("filter_stats", G_EVAL | G_NOARGS, NULL);
+	perl_call_argv("filter_stats", G_EVAL | G_NOARGS, argv);
 	SPAGAIN;
         result = POPp;
         if (result != NULL && *result)
@@ -285,6 +285,9 @@ XS(XS_INN_addhist)
     char        tbuff[32];
     char*       parambuf[6];
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items < 1 || items > 5)
         croak("Usage INN::addhist(msgid,[arrival,articletime,expire,token])");
 
@@ -300,7 +303,7 @@ XS(XS_INN_addhist)
 
     /* The token defaults to an empty string. */
     if (i == 4)
-        parambuf[4] = "";
+        parambuf[4] = (char *) "";
 
     parambuf[5] = NULL;
 
@@ -327,6 +330,9 @@ XS(XS_INN_article)
     char *      p;
     size_t      len;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
 	croak("Usage: INN::article(msgid)");
 
@@ -337,7 +343,7 @@ XS(XS_INN_article)
     /* Retrieve the article and convert it from wire format. */
     art = SMretrieve(token, RETR_ALL);
     if (art == NULL) XSRETURN_UNDEF;
-    p = FromWireFmt(art->data, art->len, &len);
+    p = wire_to_native(art->data, art->len, &len);
     SMfreearticle(art);
 
     /* Push a copy of the article onto the Perl stack, free our temporary
@@ -358,6 +364,9 @@ XS(XS_INN_cancel)
     char        *msgid;
     char        *parambuf[2];
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::cancel(msgid)");
 
@@ -384,6 +393,9 @@ XS(XS_INN_filesfor)
     char        *msgid;
     TOKEN       token;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::filesfor(msgid)");
 
@@ -404,6 +416,9 @@ XS(XS_INN_havehist)
     dXSARGS;
     char        *msgid;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::havehist(msgid)");
 
@@ -429,6 +444,9 @@ XS(XS_INN_head)
     char *      p;
     size_t      len;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::head(msgid)");
 
@@ -439,7 +457,7 @@ XS(XS_INN_head)
     /* Retrieve the article header and convert it from wire format. */
     art = SMretrieve(token, RETR_HEAD);
     if (art == NULL) XSRETURN_UNDEF;
-    p = FromWireFmt(art->data, art->len, &len);
+    p = wire_to_native(art->data, art->len, &len);
     SMfreearticle(art);
 
     /* Push a copy of the article header onto the Perl stack, free our
@@ -462,6 +480,9 @@ XS(XS_INN_newsgroup)
     char *      end;
     int         size;
 
+    /* Suppress warnings for the mandatory XS argument. */
+    cv = cv;
+
     if (items != 1)
         croak("Usage: INN::newsgroup(group)");
     newsgroup = (char *) SvPV(ST(0), PL_na);
@@ -489,13 +510,13 @@ XS(XS_INN_newsgroup)
 void
 PLxsinit(void)
 {
-    newXS("INN::addhist", XS_INN_addhist, "perl.c");
-    newXS("INN::article", XS_INN_article, "perl.c");
-    newXS("INN::cancel", XS_INN_cancel, "perl.c");
-    newXS("INN::havehist", XS_INN_havehist, "perl.c");
-    newXS("INN::head", XS_INN_head, "perl.c");
-    newXS("INN::newsgroup", XS_INN_newsgroup, "perl.c");
-    newXS("INN::filesfor", XS_INN_filesfor, "perl.c");
+    inn_newXS("INN::addhist", XS_INN_addhist, "perl.c");
+    inn_newXS("INN::article", XS_INN_article, "perl.c");
+    inn_newXS("INN::cancel", XS_INN_cancel, "perl.c");
+    inn_newXS("INN::havehist", XS_INN_havehist, "perl.c");
+    inn_newXS("INN::head", XS_INN_head, "perl.c");
+    inn_newXS("INN::newsgroup", XS_INN_newsgroup, "perl.c");
+    inn_newXS("INN::filesfor", XS_INN_filesfor, "perl.c");
 }
 
 #endif /* defined(DO_PERL) */
