@@ -1,4 +1,4 @@
-/*  $Id: innlistener.c 6716 2004-05-16 20:26:56Z rra $
+/*  $Id: innlistener.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  The implementation of the innfeed InnListener class.
 **
@@ -16,8 +16,9 @@
 #include <syslog.h>
 #include <time.h>
 
+#include "inn/libinn.h"
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/nntp.h"
 
 #include "article.h"
 #include "buffer.h"
@@ -25,7 +26,6 @@
 #include "endpoint.h"
 #include "host.h"
 #include "innlistener.h"
-#include "nntp.h"
 #include "tape.h"
 
 #define LISTENER_INPUT_BUFFER (1024 * 8) /* byte size of the input buffer */
@@ -230,8 +230,7 @@ void shutDown (InnListener l)
       char dateString [30] ;
 
       gHostStats();
-      strlcpy (dateString,ctime (&now),sizeof (dateString)) ;
-      dateString [24] = '\0' ;
+      timeToString (now, dateString, sizeof (dateString)) ;
 
       if (fastExit)
         notice ("ME finishing (quickly) at %s", dateString) ;
@@ -500,9 +499,9 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
           *fileNameEnd = ' ' ;
 
           /* Check the message ID length */
-          if (strlen(msgid) > NNTP_MSGID_MAXLEN) {
+          if (strlen(msgid) > NNTP_MAXLEN_MSGID) {
             warn ("ME message id exceeds limit of %d octets: %s",
-                  NNTP_MSGID_MAXLEN, msgid) ;
+                  NNTP_MAXLEN_MSGID, msgid) ;
             *(msgidEnd+1) = '\0' ;
           }
           *msgidEnd = ' ' ;
@@ -527,7 +526,7 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
               
               /* See if this is a valid peername */
               for(s = peer; *s; s++)
-                if (!CTYPE(isalnum, *s) && *s != '.' && *s != '-' && *s != '_')
+                if (!isalnum((unsigned char) *s) && *s != '.' && *s != '-' && *s != '_')
                   break;
               if (*s != 0) {
                   warn ("ME invalid peername %s", peer) ;
@@ -711,14 +710,11 @@ void openDroppedArticleFile (void)
 {
   pid_t myPid = getpid () ;
   const char *tapeDir = getTapeDirectory() ;
-  size_t len;
 
   if (dropArtFile != NULL)
     free (dropArtFile) ;
 
-  len = pathMax(tapeDir) + 1;
-  dropArtFile = xmalloc(len);
-  snprintf (dropArtFile,len,"%s/innfeed-dropped.%c%06d",
+  xasprintf (&dropArtFile, "%s/innfeed-dropped.%c%06d",
             tapeDir, droppedFileCount + 'A', (int) myPid) ;
 
   if ((droppedFp = fopen (dropArtFile,"w")) == NULL)
