@@ -1,4 +1,4 @@
-/*  $Id: tape.c 6716 2004-05-16 20:26:56Z rra $
+/*  $Id: tape.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  The implementation of the innfeed Tape class.
 **
@@ -27,13 +27,8 @@
 #include <sys/stat.h>
 #include <syslog.h>
 
-#if defined (HAVE_DIRENT_H)
 #include <dirent.h>
 typedef struct dirent DIRENTRY ;
-#else
-#include <sys/dir.h>
-typedef struct direct DIRENTRY ;
-#endif
 
 #ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -48,7 +43,7 @@ typedef struct direct DIRENTRY ;
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 #include "article.h"
 #include "configfile.h"
@@ -185,7 +180,7 @@ int tapeConfigLoadCbk (void *data)
 
   if (getString (topScope,"backlog-directory",&p,NO_INHERIT))
     {
-      dir = buildFilename(innconf->pathspool, p);
+      dir = concatpath(innconf->pathspool, p);
       free(p);
       if (tapeDirectory != NULL && strcmp (tapeDirectory,dir) != 0)
         {
@@ -640,6 +635,7 @@ void tapeTakeArticle (Tape tape, Article article)
   const char *fname, *msgid ;
 
   ASSERT (tape != NULL) ;
+  ASSERT (article != NULL) ;
 
   /* return immediately if spooling disabled - jgarzik */
   if (tape->noBacklog)
@@ -738,7 +734,7 @@ Article getArticle (Tape tape)
         {
           msgid = filename = NULL ;
           
-          for (p = line ; *p && CTYPE(isspace, *p) ; p++) /* eat whitespace */
+          for (p = line ; *p && isspace((unsigned char) *p) ; p++) /* eat whitespace */
             /* nada */ ;
 
           if (*p != '\0')
@@ -750,7 +746,7 @@ Article getArticle (Tape tape)
                   filename = p ;
                   *q = '\0' ;
       
-                  for (q++ ; *q && CTYPE(isspace, *q) ; q++)
+                  for (q++ ; *q && isspace((unsigned char) *q) ; q++)
                     /* nada */ ;
 
                   if (*q != '\0')
