@@ -1,23 +1,23 @@
-/*  $Id: date.c,v 1.12 2003/01/19 01:15:33 rra Exp $
+/*  $Id: date.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Date parsing and conversion routines.
 **
 **  Provides various date parsing and conversion routines, including
-**  generating Date headers for posted articles.  Note that the parsedate
-**  parser is separate from this file.
+**  generating Date headers for posted articles.
 */
 
 #include "config.h"
 #include "clibrary.h"
+#include "headers.c"
 #include <ctype.h>
 #include <time.h>
 
-#include "libinn.h"
+#include "inn/libinn.h"
 
 /*
 **  Time constants.
 **
-**  Do not translate these names.  RFC 822 by way of RFC 1036 requires that
+**  Do not translate these names.  RFC 5322 by way of RFC 5536 requires that
 **  weekday and month names *not* be translated.  This is why we use static
 **  tables rather than strftime for building dates, to avoid locale
 **  interference.
@@ -32,13 +32,19 @@ static const char MONTH[12][4] = {
     "Nov", "Dec"
 };
 
+/* Complete month names, used only for lax date parsing. */
+static const char OBS_MONTH[12][10] = {
+    "January",  "February",  "March",     "April",     "May",       "June",
+    "July",     "August",    "September", "October",   "November",  "December"
+};
+
 /* Number of days in a month. */
 static const int MONTHDAYS[] = {
     31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
 };
 
 /* Non-numeric time zones.  Supporting these is required to support the
-   obsolete date format of RFC 2822.  The military time zones are handled
+   obsolete date format of RFC 5322.  The military time zones are handled
    separately. */
 static const struct {
     const char name[4];
@@ -51,6 +57,52 @@ static const struct {
     { "PDT", -7 * 60 * 60 },    { "PST", -8 * 60 * 60 },
 };
 
+/* Additional non-numeric time zones supported because the old parsedate
+   parser supported them.  These aren't legal in RFC 5322, but are supported
+   in lax mode. */
+static const struct {
+    const char name[5];
+    long offset;
+} OBS_ZONE_OFFSET[] = {
+    { "UTC",    0 },                 /* Universal Coordinated */
+    { "CUT",    0 },                 /* Coordinated Universal */
+    { "WET",    0 },                 /* Western European */
+    { "BST",    1 * 60 * 60 },       /* British Summer */
+    { "NDT",  (-2 * 60 + 30) * 60 }, /* Newfoundland Daylight */
+    { "NST",  (-3 * 60 + 30) * 60 }, /* Newfoundland Standard */
+    { "ADT",   -3 * 60 * 60 },       /* Atlantic Daylight */
+    { "AST",   -4 * 60 * 60 },       /* Atlantic Standard */
+    { "YDT",   -8 * 60 * 60 },       /* Yukon Daylight */
+    { "YST",   -9 * 60 * 60 },       /* Yukon Standard */
+    { "AKDT",  -8 * 60 * 60 },       /* Alaska Daylight */
+    { "AKST",  -9 * 60 * 60 },       /* Alaska Standard */
+    { "HADT",  -9 * 60 * 60 },       /* Hawaii-Aleutian Daylight */
+    { "HAST", -10 * 60 * 60 },       /* Hawaii-Aleutian Standard */
+    { "HST",  -10 * 60 * 60 },       /* Hawaii Standard */
+    { "CES",    2 * 60 * 60 },       /* Central European Summer */
+    { "CEST",   2 * 60 * 60 },       /* Central European Summer */
+    { "MEZ",    1 * 60 * 60 },       /* Middle European */
+    { "MEZT",   2 * 60 * 60 },       /* Middle European Summer */
+    { "CET",    1 * 60 * 60 },       /* Central European */
+    { "MET",    1 * 60 * 60 },       /* Middle European */
+    { "EET",    2 * 60 * 60 },       /* Eastern European */
+    { "MSK",    3 * 60 * 60 },       /* Moscow Winter */
+    { "MSD",    4 * 60 * 60 },       /* Moscow Summer */
+    { "WAST",   8 * 60 * 60 },       /* Western Australian Standard */
+    { "WADT",   9 * 60 * 60 },       /* Western Australian Daylight */
+    { "HKT",    8 * 60 * 60 },       /* Hong Kong */
+    { "CCT",    8 * 60 * 60 },       /* China Coast */
+    { "JST",    9 * 60 * 60 },       /* Japan Standard */
+    { "KST",    9 * 60 * 60 },       /* Korean Standard */
+    { "KDT",    9 * 60 * 60 },       /* Korean Daylight (no change?) */
+    { "CAST",  (9 * 60 + 30) * 60 }, /* Central Australian Standard */
+    { "CADT", (10 * 60 + 30) * 60 }, /* Central Australian Daylight */
+    { "EAST",  10 * 60 * 60 },       /* Eastern Australian Standard */
+    { "EADT",  11 * 60 * 60 },       /* Eastern Australian Daylight */
+    { "NZST",  12 * 60 * 60 },       /* New Zealand Standard */
+    { "NZST",  13 * 60 * 60 },       /* New Zealand Daylight */
+};
+
 
 /*
 **  Time parsing macros.
@@ -62,7 +114,7 @@ static const struct {
 
 
 /*
-**  RFC 2822 date parsing rules.
+**  RFC 5322 date parsing rules.
 */
 
 /* The data structure to store a rule.  The interpretation of the other fields
@@ -74,6 +126,7 @@ struct rule {
     enum {
         TYPE_NUMBER,
         TYPE_LOOKUP,
+        TYPE_OBS_MONTH,
         TYPE_DELIM
     } type;
     char delimiter;
@@ -95,19 +148,19 @@ static long
 local_tz_offset(time_t date, bool current UNUSED)
 {
     struct tm *tm;
-#if !HAVE_TM_GMTOFF
+#if !HAVE_STRUCT_TM_TM_GMTOFF
     struct tm local, gmt;
     long offset;
 #endif
 
     tm = localtime(&date);
 
-#if !HAVE_TM_GMTOFF && HAVE_VAR_TIMEZONE
+#if !HAVE_STRUCT_TM_TM_GMTOFF && HAVE_DECL_ALTZONE
     if (current)
         return (tm->tm_isdst > 0) ? -altzone : -timezone;
 #endif
 
-#if HAVE_TM_GMTOFF
+#if HAVE_STRUCT_TM_TM_GMTOFF
     return tm->tm_gmtoff;
 #else
     /* We don't have any easy returnable value, so we call both localtime
@@ -136,8 +189,8 @@ local_tz_offset(time_t date, bool current UNUSED)
 
 /*
 **  Given a time_t, a flag saying whether to use local time, a buffer, and
-**  the length of the buffer, write the contents of a valid RFC 2822 / RFC
-**  1036 Date header into the buffer (provided it's long enough).  Returns
+**  the length of the buffer, write the contents of a valid RFC 5322 / RFC
+**  5536 Date header into the buffer (provided it's long enough).  Returns
 **  true on success, false if the buffer is too long.  Use snprintf rather
 **  than strftime to be absolutely certain that locales don't result in the
 **  wrong output.  If the time is -1, obtain and use the current time.
@@ -153,7 +206,7 @@ makedate(time_t date, bool local, char *buff, size_t buflen)
     size_t date_length;
     const char *tz_name;
 
-    /* Make sure the buffer is large enough.  A complete RFC 2822 date with
+    /* Make sure the buffer is large enough.  A complete RFC 5322 date with
        spaces wherever FWS is required and the optional weekday takes:
 
                     1         2         3
@@ -161,15 +214,15 @@ makedate(time_t date, bool local, char *buff, size_t buflen)
            Sat, 31 Aug 2002 23:45:18 +0000
 
        31 characters, plus another character for the trailing nul.  The buffer
-       will need to have another six characters of space to get the optional
-       trailing time zone comment. */
+       will need to have at least another six characters of space to get the
+       optional trailing time zone comment. */
     if (buflen < 32)
         return false;
 
     /* Get the current time if the provided time is -1. */
     realdate = (date == (time_t) -1) ? time(NULL) : date;
 
-    /* RFC 2822 says the timezone offset is given as [+-]HHMM, so we have to
+    /* RFC 5322 says the timezone offset is given as [+-]HHMM, so we have to
        separate the offset into a sign, hours, and minutes.  Dividing the
        offset by 36 looks like it works, but will fail for any offset that
        isn't an even number of hours, and there are half-hour timezones. */
@@ -210,9 +263,9 @@ makedate(time_t date, bool local, char *buff, size_t buflen)
     if (!local) {
         tz_name = "UTC";
     } else {
-#if HAVE_TM_ZONE
+#if HAVE_STRUCT_TM_TM_ZONE
         tz_name = tm.tm_zone;
-#elif HAVE_VAR_TZNAME
+#elif HAVE_TZNAME
         tz_name = tzname[(tm.tm_isdst > 0) ? 1 : 0];
 #else
         tz_name = NULL;
@@ -304,16 +357,15 @@ parsedate_nntp(const char *date, const char *hour, bool local)
     struct tm *current;
     int century;
 
-    /* Accept YYMMDD and YYYYMMDD.  The first is what RFC 977 requires.  The
-       second is what the revision of RFC 977 will require. */
+    /* Accept YYMMDD and YYYYMMDD. */
     datelen = strlen(date);
     if ((datelen != 6 && datelen != 8) || strlen(hour) != 6)
         return (time_t) -1;
     for (p = date; *p; p++)
-        if (!CTYPE(isdigit, *p))
+        if (!isdigit((unsigned char) *p))
             return (time_t) -1;
     for (p = hour; *p; p++)
-        if (!CTYPE(isdigit, *p))
+        if (!isdigit((unsigned char) *p))
             return (time_t) -1;
 
     /* Parse the date into a struct tm, skipping over the century part of
@@ -330,15 +382,11 @@ parsedate_nntp(const char *date, const char *hour, bool local)
 
     /* Four-digit years are the easy case.
 
-       For two-digit years, RFC 977 says "The closest century is assumed as
-       part of the year (i.e., 86 specifies 1986, 30 specifies 2030, 99 is
-       1999, 00 is 2000)."  draft-ietf-nntpext-base-10.txt simplifies this
-       considerably and is what we implement:
+       For two-digit years, RFC 3977 says:
 
          If the first two digits of the year are not specified, the year is
          to be taken from the current century if YY is smaller than or equal
-         to the current year, otherwise the year is from the previous
-         century.
+         to the current year, and the previous century otherwise.
 
        This implementation assumes "current year" means the last two digits
        of the current year.  Note that this algorithm interacts poorly with
@@ -371,53 +419,6 @@ parsedate_nntp(const char *date, const char *hour, bool local)
 }
 
 
-/*
-**  Skip any amount of CFWS (comments and folding whitespace), the RFC 2822
-**  grammar term for whitespace, CRLF pairs, and possibly nested comments that
-**  may contain escaped parens.  We also allow simple newlines since we don't
-**  always deal with wire-format messages.  Note that we do not attempt to
-**  ensure that CRLF or a newline is followed by whitespace.  Returns the new
-**  position of the pointer.
-*/
-static const char *
-skip_cfws(const char *p)
-{
-    int nesting = 0;
-
-    for (; *p != '\0'; p++) {
-        switch (*p) {
-        case ' ':
-        case '\t':
-        case '\n':
-            break;
-        case '\r':
-            if (p[1] != '\n')
-                return p;
-            p++;
-            break;
-        case '(':
-            nesting++;
-            break;
-        case ')':
-            if (nesting == 0)
-                return p;
-            nesting--;
-            break;
-        case '\\':
-            if (nesting == 0 || p[1] == '\0')
-                return p;
-            p++;
-            break;
-        default:
-            if (nesting == 0)
-                return p;
-            break;
-        }
-    }
-    return p;
-}
-
-
 /*
 **  Parse a single number.  Takes the parsing rule that we're applying and
 **  returns a pointer to the new position of the parse stream.  If there
@@ -461,6 +462,49 @@ parse_lookup(const char *p, const struct rule *rule, int *value)
 }
 
 
+/*
+**  Parse a single string value that should be an obsolete month name.  If we
+**  have three characters available, check against the abbreviation.  Likewise
+**  if we have four characters and the fourth is a period.  Otherwise, check
+**  against the full English month name.  Puts the month number into the value
+**  pointer and and returns the new position of the string, or NULL if the
+**  string is not a valid month.
+*/
+static const char *
+parse_legacy_month(const char *p, const struct rule *rule UNUSED, int *value)
+{
+    size_t i, size;
+    const char *end;
+
+    for (end = p; *end != '\0' && isalpha((unsigned char) *end); end++)
+        ;
+    if (*end == '.')
+        end++;
+    if (end == p)
+        return NULL;
+    size = end - p;
+    if (size == 3 || (size == 4 && p[3] == '.')) {
+        for (i = 0; i < ARRAY_SIZE(MONTH); i++)
+            if (strncasecmp(MONTH[i], p, 3) == 0) {
+                p += size;
+                *value = i;
+                return p;
+            }
+    } else {
+        for (i = 0; i < ARRAY_SIZE(OBS_MONTH); i++) {
+            if (size != strlen(OBS_MONTH[i]))
+                continue;
+            if (strncasecmp(OBS_MONTH[i], p, size) == 0) {
+                p += size;
+                *value = i;
+                return p;
+            }
+        }
+    }
+    return NULL;
+}
+
+
 /*
 **  Apply a set of date parsing rules to a string.  Returns the new position
 **  in the parse string if this succeeds and NULL if it fails.  As part of the
@@ -485,16 +529,17 @@ parse_by_rule(const char *p, const struct rule rules[], size_t count,
             break;
         case TYPE_LOOKUP:
             p = parse_lookup(p, rule, &values[i]);
-            if (p == NULL)
-                return NULL;
+            break;
+        case TYPE_OBS_MONTH:
+            p = parse_legacy_month(p, rule, &values[i]);
             break;
         case TYPE_NUMBER:
             p = parse_number(p, rule, &values[i]);
-            if (p == NULL)
-                return NULL;
             break;
         }
 
+        if (p == NULL)
+            return NULL;
         p = skip_cfws(p);
     }
     return p;
@@ -502,18 +547,19 @@ parse_by_rule(const char *p, const struct rule rules[], size_t count,
 
 
 /*
-**  Parse a legacy time zone.  This uses the parsing rules in RFC 2822,
+**  Parse a legacy time zone.  This uses the parsing rules in RFC 5322,
 **  including assigning an offset of 0 to all single-character military time
 **  zones due to their ambiguity in practice.  Returns the new position in the
-**  parse stream or NULL if we failed to parse the zone.
+**  parse stream or NULL if we failed to parse the zone.  If the obsolete flag
+**  is set, also check against obsolete time zones.
 */
 static const char *
-parse_legacy_timezone(const char *p, long *offset)
+parse_legacy_timezone(const char *p, long *offset, bool obsolete)
 {
     const char *end;
     size_t max, i;
 
-    for (end = p; *end != '\0' && !CTYPE(isspace, *end); end++)
+    for (end = p; *end != '\0' && isalpha((unsigned char) *end); end++)
         ;
     if (end == p)
         return NULL;
@@ -524,51 +570,62 @@ parse_legacy_timezone(const char *p, long *offset)
             *offset = ZONE_OFFSET[i].offset;
             return p;
         }
-    if (max == 1 && CTYPE(isalpha, *p) && *p != 'J' && *p != 'j') {
+    if (max == 1 && isalpha((unsigned char) *p) && *p != 'J' && *p != 'j') {
         *offset = 0;
         return p + 1;
     }
+    if (obsolete)
+        for (i = 0; i < ARRAY_SIZE(OBS_ZONE_OFFSET); i++) {
+            if (strlen(OBS_ZONE_OFFSET[i].name) > max)
+                continue;
+            if (strncasecmp(OBS_ZONE_OFFSET[i].name, p, max) == 0) {
+                p += strlen(OBS_ZONE_OFFSET[i].name);
+                *offset = OBS_ZONE_OFFSET[i].offset;
+                return p;
+            }
+        }
     return NULL;
 }
 
 
 /*
-**  Parse an RFC 2822 date, accepting the normal and obsolete syntax.  Takes a
-**  pointer to the beginning of the date and the length.  Returns the
-**  translated time in seconds since epoch, or (time_t) -1 on error.
+**  Parse an RFC 5322 date, accepting the normal and obsolete syntax.  Takes a
+**  pointer to the beginning of the date.  Returns the translated time in
+**  seconds since epoch, or (time_t) -1 on error.
 */
 time_t
-parsedate_rfc2822(const char *date)
+parsedate_rfc5322(const char *date)
 {
     const char *p;
     int zone_sign;
     long zone_offset;
     struct tm tm;
-    int values[7];
+    int values[8];
     time_t result;
 
     /* The basic rules.  Note that we don't bother to check whether the day of
        the week is accurate or not. */
     static const struct rule base_rule[] = {
-        { TYPE_LOOKUP, 0,   WEEKDAY, 7,  3, 3 },
-        { TYPE_DELIM,  ',', NULL,    0,  1, 1 },
-        { TYPE_NUMBER, 0,   NULL,    0,  1, 2 },
+        { TYPE_LOOKUP, 0,   WEEKDAY,  7, 3, 3 },
+        { TYPE_DELIM,  ',', NULL,     0, 1, 1 },
+        { TYPE_NUMBER, 0,   NULL,     0, 1, 2 },
         { TYPE_LOOKUP, 0,   MONTH,   12, 3, 3 },
-        { TYPE_NUMBER, 0,   NULL,    0,  2, 4 },
-        { TYPE_NUMBER, 0,   NULL,    0,  2, 2 },
-        { TYPE_DELIM,  ':', NULL,    0,  1, 1 },
-        { TYPE_NUMBER, 0,   NULL,    0,  2, 2 }
+        { TYPE_NUMBER, 0,   NULL,     0, 2, 4 },
+        { TYPE_NUMBER, 0,   NULL,     0, 2, 2 },
+        { TYPE_DELIM,  ':', NULL,     0, 1, 1 },
+        { TYPE_NUMBER, 0,   NULL,     0, 2, 2 }
     };
 
     /* Optional seconds at the end of the time. */
     static const struct rule seconds_rule[] = {
-        { TYPE_DELIM,  ':', NULL,    0,  1, 1 },
-        { TYPE_NUMBER, 0,   NULL,    0,  2, 2 }
+        { TYPE_DELIM,  ':', NULL,     0, 1, 1 },
+        { TYPE_NUMBER, 0,   NULL,     0, 2, 2 }
     };
 
-    /* Numeric time zone. */
+    /* Numeric time zone.  Keep the hours and minutes separate. */
     static const struct rule zone_rule[] = {
-        { TYPE_NUMBER, 0,   NULL,    0,  4, 4 }
+        { TYPE_NUMBER, 0,   NULL,     0, 2, 2 },
+        { TYPE_NUMBER, 0,   NULL,     0, 2, 2 }
     };
 
     /* Start with a clean slate. */
@@ -578,7 +635,7 @@ parsedate_rfc2822(const char *date)
     /* Parse the base part of the date.  The initial day of the week is
        optional. */
     p = skip_cfws(date);
-    if (CTYPE(isalpha, *p))
+    if (isalpha((unsigned char) *p))
         p = parse_by_rule(p, base_rule, ARRAY_SIZE(base_rule), values);
     else
         p = parse_by_rule(p, base_rule + 2, ARRAY_SIZE(base_rule) - 2,
@@ -609,15 +666,15 @@ parsedate_rfc2822(const char *date)
         p = parse_by_rule(p + 1, zone_rule, ARRAY_SIZE(zone_rule), values);
         if (p == NULL)
             return (time_t) -1;
-        zone_offset = ((values[0] / 100) * 60 + values[0] % 100) * 60;
+        zone_offset = (values[0] * 60 + values[1]) * 60;
         zone_offset *= zone_sign;
     } else {
-        p = parse_legacy_timezone(p, &zone_offset);
+        p = parse_legacy_timezone(p, &zone_offset, false);
         if (p == NULL)
             return (time_t) -1;
     }
 
-    /* Fix up the year, using the RFC 2822 rules.  Remember that tm_year
+    /* Fix up the year, using the RFC 5322 rules.  Remember that tm_year
        stores the year - 1900. */
     if (tm.tm_year < 50)
         tm.tm_year += 100;
@@ -635,3 +692,128 @@ parsedate_rfc2822(const char *date)
     result = mktime_utc(&tm);
     return (result == (time_t) -1) ? result : result - zone_offset;
 }
+
+
+/*
+**  Parse a date, accepting a lax syntax that tries to allow for any vaguely
+**  RFC-5322-like date that the old parsedate code would accept, allowing for
+**  a lot of variation that's seen in Usenet articles.  Takes a pointer to the
+**  date and returns the translated time in seconds since epoch, or (time_t)
+**  -1 on error.
+*/
+time_t
+parsedate_rfc5322_lax(const char *date)
+{
+    const char *p, *start;
+    struct tm tm;
+    int values[6];
+    bool have_zone;
+    int zone_sign;
+    long zone_offset;
+    time_t result;
+
+    /* The basic rules.  Allow one or two digits in time components, since
+       some broken software omits the leading zero and parsedate didn't
+       care. */
+    static const struct rule base_rule[] = {
+        { TYPE_NUMBER,    0,   NULL,  0, 1, 2 },
+        { TYPE_OBS_MONTH, 0,   NULL, 12, 3, 3 },
+        { TYPE_NUMBER,    0,   NULL,  0, 2, 4 },
+        { TYPE_NUMBER,    0,   NULL,  0, 1, 2 },
+        { TYPE_DELIM,     ':', NULL,  0, 1, 1 },
+        { TYPE_NUMBER,    0,   NULL,  0, 1, 2 }
+    };
+
+    /* Optional seconds at the end of the time.  Similarly, don't require the
+       leading zero. */
+    static const struct rule seconds_rule[] = {
+        { TYPE_DELIM,     ':', NULL,  0, 1, 1 },
+        { TYPE_NUMBER,    0,   NULL,  0, 1, 2 }
+    };
+
+    /* Numeric time zone.  Allow the hour portion to omit the leading zero.
+       Unfortunately, our parser is greedy, so we have to parse this as one
+       number and then patch it up later. */
+    static const struct rule zone_rule[] = {
+        { TYPE_NUMBER,    0,   NULL,  0, 1, 5 }
+    };
+
+    /* Start with a clean slate. */
+    memset(&tm, 0, sizeof(struct tm));
+    memset(values, 0, sizeof(values));
+
+    /* Parse the base part of the date.  The initial day of the week is
+       optional.  Allow for anything that looks vaguely day-like. */
+    p = skip_cfws(date);
+    while (*p != '\0' && !isdigit((unsigned char) *p) && *p != ',')
+        p++;
+    if (*p == ',')
+        p = skip_cfws(p + 1);
+    p = parse_by_rule(p, base_rule, ARRAY_SIZE(base_rule), values);
+    if (p == NULL)
+        return (time_t) -1;
+
+    /* Stash the results into a struct tm.  Values are associated with the
+       rule number of the same index. */
+    tm.tm_mday = values[0];
+    tm.tm_mon = values[1];
+    tm.tm_year = values[2];
+    tm.tm_hour = values[3];
+    tm.tm_min = values[5];
+
+    /* Parse seconds if they're present. */
+    if (*p == ':') {
+        p = parse_by_rule(p, seconds_rule, ARRAY_SIZE(seconds_rule), values);
+        if (p == NULL)
+            return (time_t) -1;
+        tm.tm_sec = values[1];
+    }
+
+    /* Time zone.  Unfortunately this is weird enough that we can't use nice
+       parsing rules for it.  If we don't recognize the time zone at all, just
+       bail and assume GMT.  parsedate used the final time zone found, when
+       multiple ones were supplied, so emulate that behavior. */
+    have_zone = false;
+    zone_offset = 0;
+    while (p != NULL && *p != '\0') {
+        if (*p == '-' || *p == '+') {
+            zone_sign = (*p == '+') ? 1 : -1;
+            start = p + 1;
+            p = parse_by_rule(start, zone_rule, ARRAY_SIZE(zone_rule), values);
+            if (p == NULL)
+                return (time_t) -1;
+            if (p - start < 3)
+                zone_offset = values[0] * 60 * 60;
+            else
+                zone_offset = ((values[0] / 100) * 60 + values[0] % 100) * 60;
+            zone_offset *= zone_sign;
+        } else {
+            p = parse_legacy_timezone(p, &zone_offset, true);
+            if (p == NULL)
+                zone_offset = 0;
+        }
+        have_zone = true;
+        if (p != NULL)
+            p = skip_cfws(p);
+    }
+
+    /* Fix up the year, using the RFC 5322 rules.  Remember that tm_year
+       stores the year - 1900. */
+    if (tm.tm_year < 50)
+        tm.tm_year += 100;
+    else if (tm.tm_year >= 1000)
+        tm.tm_year -= 1900;
+
+    /* Done parsing.  We don't check to see if this is the end of the string;
+       we allow for any trailing garbage.  Convert the struct tm to seconds
+       since epoch and then apply the time zone offset. */
+    if (!valid_tm(&tm))
+        return (time_t) -1;
+    if (have_zone)
+        result = mktime_utc(&tm);
+    else {
+        tm.tm_isdst = -1;
+        result = mktime(&tm);
+    }
+    return (result == (time_t) -1) ? result : result - zone_offset;
+}
