@@ -1,4 +1,4 @@
-/*  $Id: getmodaddr.c 6155 2003-01-19 19:58:25Z rra $
+/*  $Id: getmodaddr.c 8921 2010-01-22 23:33:08Z iulius $
 **
 */
 
@@ -7,9 +7,9 @@
 #include <errno.h>
 
 #include "inn/innconf.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 
 
 static char	*GMApathname = NULL;
@@ -40,6 +40,7 @@ static FILE *
 GMA_listopen(int fd, FILE *FromServer, FILE *ToServer, const char *request)
 {
     char	buff[BUFSIZ];
+    char        expectedanswer[BUFSIZ];
     char	*p;
     int		oerrno;
     FILE	*F;
@@ -50,14 +51,16 @@ GMA_listopen(int fd, FILE *FromServer, FILE *ToServer, const char *request)
 
     /* Send a LIST command to and capture the output. */
     if (request == NULL)
-	fprintf(ToServer, "list moderators\r\n");
+	fprintf(ToServer, "LIST MODERATORS\r\n");
     else
-	fprintf(ToServer, "list %s\r\n", request);
+	fprintf(ToServer, "LIST %s\r\n", request);
     fflush(ToServer);
 
+    snprintf(expectedanswer, sizeof(expectedanswer), "%d", NNTP_OK_LIST);
+
     /* Get the server's reply to our command. */
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || strncmp(buff, NNTP_LIST_FOLLOWS, strlen(NNTP_LIST_FOLLOWS)) != 0) {
+     || strncmp(buff, expectedanswer, strlen(expectedanswer)) != 0) {
 	oerrno = errno;
         fclose(F);
 	GMAclose();
@@ -111,23 +114,23 @@ GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group,
          *  This should be part of nnrpd or the like running on the server.
          *  Open the server copy of the moderators file.
          */
-        path = concatpath(innconf->pathetc, _PATH_MODERATORS);
+        path = concatpath(innconf->pathetc, INN_PATH_MODERATORS);
 	GMAfp = fopen(path, "r");
         free(path);
     }else{
         /*
          *  Get a local copy of the moderators file from the server.
          */
-        GMApathname = concatpath(innconf->pathtmp, _PATH_TEMPMODERATORS);
+        GMApathname = concatpath(innconf->pathtmp, INN_PATH_TEMPMODERATORS);
         fd = mkstemp(GMApathname);
         if (fd >= 0)
-            GMAfp = GMA_listopen(fd, FromServer, ToServer, "moderators");
+            GMAfp = GMA_listopen(fd, FromServer, ToServer, "MODERATORS");
         else
             GMAfp = NULL;
 
 	/* Fallback to the local copy if the server doesn't have it */
 	if (GMAfp == NULL) {
-            path = concatpath(innconf->pathetc, _PATH_MODERATORS);
+            path = concatpath(innconf->pathetc, INN_PATH_MODERATORS);
 	    GMAfp = fopen(path, "r");
             free(path);
         }
