@@ -1,4 +1,4 @@
-/*  $Id: inndcomm.c 8307 2009-01-31 08:30:38Z iulius $
+/*  $Id: inndcomm.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Library routines to let other programs control innd.
 */
@@ -396,8 +396,8 @@ ICCcommand(char cmd, const char *argv[], char **replyp)
     /* Parse the rest of the reply; expected to be like
        <exitcode><space><text>" */
     i = 0;
-    if (CTYPE(isdigit, buff[0])) {
-	for (p = buff; *p && CTYPE(isdigit, *p); p++)
+    if (isdigit((unsigned char) buff[0])) {
+	for (p = buff; *p && isdigit((unsigned char) *p); p++)
 	    continue;
 	if (*p) {
 	    save = *p;
