@@ -1,4 +1,4 @@
-/*  $Id: perm.c 8271 2008-12-31 21:40:28Z iulius $
+/*  $Id: perm.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  How to figure out where a user comes from, and what that user can do once
 **  we know who sie is.
@@ -677,7 +677,7 @@ authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
          * therefore also downcase the wildmat patterns to make sure there
          * aren't any surprises.  DNS is case-insensitive. */
         for (p = curauth->hosts; *p; p++)
-            if (CTYPE(isupper, (unsigned char) *p))
+            if (isupper((unsigned char) *p))
                 *p = tolower((unsigned char) *p);
 
 	break;
@@ -958,7 +958,7 @@ accessdecl_parse(ACCESSGROUP *curaccess, CONFFILE *f, CONFTOKEN *tok)
 	SET_CONFIG(oldtype);
 	break;
       case PERMlocalmaxartsize:
-	curaccess->localmaxartsize = atol(tok->name);
+	curaccess->localmaxartsize = strtoul(tok->name, NULL, 10);
 	SET_CONFIG(oldtype);
 	break;
       case PERMreadertrack:
@@ -984,7 +984,7 @@ accessdecl_parse(ACCESSGROUP *curaccess, CONFFILE *f, CONFTOKEN *tok)
 	SET_CONFIG(oldtype);
 	break;
       case PERMnnrpdpostport:
-	curaccess->nnrpdpostport = atoi(tok->name);
+	curaccess->nnrpdpostport = strtoul(tok->name, NULL, 10);
 	SET_CONFIG(oldtype);
 	break;
       case PERMnnrpdoverstats:
@@ -1002,19 +1002,19 @@ accessdecl_parse(ACCESSGROUP *curaccess, CONFFILE *f, CONFTOKEN *tok)
 	SET_CONFIG(oldtype);
 	break;
       case PERMbackoff_k:
-	curaccess->backoff_k = atol(tok->name);
+	curaccess->backoff_k = strtoul(tok->name, NULL, 10);
 	SET_CONFIG(oldtype);
 	break;
       case PERMbackoff_postfast:
-	curaccess->backoff_postfast = atol(tok->name);
+	curaccess->backoff_postfast = strtoul(tok->name, NULL, 10);
 	SET_CONFIG(oldtype);
 	break;
       case PERMbackoff_postslow:
-	curaccess->backoff_postslow = atol(tok->name);
+	curaccess->backoff_postslow = strtoul(tok->name, NULL, 10);
 	SET_CONFIG(oldtype);
 	break;
       case PERMbackoff_trigger:
-	curaccess->backoff_trigger = atol(tok->name);
+	curaccess->backoff_trigger = strtoul(tok->name, NULL, 10);
 	SET_CONFIG(oldtype);
 	break;
       case PERMnnrpdcheckart:
@@ -1718,10 +1718,10 @@ PERMgetpermissions(void)
 	    }
 	    if (VirtualPath)
 		free(VirtualPath);
-	    if (strcmp(innconf->pathhost, PERMaccessconf->pathhost) == 0) {
+	    if (strcasecmp(innconf->pathhost, PERMaccessconf->pathhost) == 0) {
 		/* Use domain, if pathhost in access realm matches one in
 		 * inn.conf to differentiate virtual host. */
-		if (innconf->domain != NULL && strcmp(innconf->domain, PERMaccessconf->domain) == 0) {
+		if (innconf->domain != NULL && strcasecmp(innconf->domain, PERMaccessconf->domain) == 0) {
 		    syslog(L_ERROR, "%s domain parameter (%s) in readers.conf must be different from the one in inn.conf.",
 			Client.host, PERMaccessconf->name);
 		    Reply("%d NNTP server unavailable.  Try later!\r\n", NNTP_FAIL_TERMINATING);
