@@ -1,4 +1,4 @@
-/* $Id: xmalloc.c 7585 2006-11-21 09:37:51Z eagle $ */
+/* $Id: xmalloc.c 9024 2010-03-21 16:49:30Z iulius $ */
 /* Test suite for xmalloc and family. */
 
 #include "config.h"
@@ -246,7 +246,7 @@ main(int argc, char *argv[])
 
     /* If the code is capitalized, install our customized error handler. */
     code = argv[1][0];
-    if (isupper(code)) {
+    if (isupper((unsigned char) code)) {
         xmalloc_error_handler = test_handler;
         code = tolower(code);
     }
